@@ -36,18 +36,22 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#else
+#define HAVE_CONFIG_H
+
 /*****************************************************************************
  * Program information
  *****************************************************************************/
 
 /* Program version and copyright message */
-#define VERSION_MESSAGE     "vlc 0.2.91 Ourumov " \
+#define VERSION_MESSAGE     "vlc 0.2.92-pre1 Ourumov " \
                           /* "(" PROGRAM_BUILD ") (" PROGRAM_OPTIONS ")\n" */ \
                             "Copyright 1996-2001 VideoLAN\n"
-#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.91" \
+#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.92-pre1" \
                             " Ourumov - (C)1996-2001 VideoLAN"
 
-#define VERSION             "0.2.91"
+#define VERSION             "0.2.92-pre1"
 
 /*****************************************************************************
  * Debugging options - define or undefine symbols
@@ -86,6 +90,13 @@
  * status */
 #define THREAD_SLEEP                    ((int)(0.010*CLOCK_FREQ))
 
+/* When a thread waits on a condition in debug mode, delay to wait before
+ * outputting an error message (in second) */
+#define THREAD_COND_TIMEOUT				5
+
+/* Environment variable containing the memcpy method */
+#define MEMCPY_METHOD_VAR               "vlc_memcpy"
+
 /*
  * Decoders FIFO configuration
  */
@@ -168,22 +179,23 @@
 #define INPUT_NETWORK_PROTOCOL_VAR      "vlc_network_protocol"
 #define INPUT_NETWORK_PROTOCOL_DEFAULT  "ts"
 
-/* Default remote server */
-#define INPUT_SERVER_VAR                "vlc_server"
-#define INPUT_SERVER_DEFAULT            "138.195.143.220"
-
 /* Default input port */
 #define INPUT_PORT_VAR                  "vlc_server_port"
 #define INPUT_PORT_DEFAULT              1234
 
-/* Default broadcast address */
-#define INPUT_BCAST_ADDR_VAR            "vlc_broadcast_addr"
-#define INPUT_BCAST_ADDR_DEFAULT        "138.195.143.255"
+/* FIXME : Delete those ! */
+/* Default remote server */
+#define INPUT_SERVER_VAR                "vlc_server"
+#define INPUT_SERVER_DEFAULT            "138.195.143.220"
 
 /* Broadcast mode */
 #define INPUT_BROADCAST_VAR             "vlc_broadcast"
 #define INPUT_BROADCAST_DEFAULT         0
 
+/* Default broadcast address */
+#define INPUT_BCAST_ADDR_VAR            "vlc_broadcast_addr"
+#define INPUT_BCAST_ADDR_DEFAULT        "138.195.143.255"
+
 /* Channels mode */
 #define INPUT_NETWORK_CHANNEL_VAR       "vlc_channel"
 #define INPUT_NETWORK_CHANNEL_DEFAULT   0
@@ -292,6 +304,9 @@
  * - long, in order to perform the buffer calculations as few as possible */
 #define AOUT_BUFFER_DURATION            100000
 
+/* Environment variable for MPEG audio decoder */
+#define ADEC_MPEG_VAR                   "vlc_mpeg_adec"
+
 /*****************************************************************************
  * Video configuration
  *****************************************************************************/
@@ -322,19 +337,12 @@
 /* Default dimensions for display window - these dimensions are enough for the
  * standard width and height broadcasted MPEG-2 streams or DVDs */
 #define VOUT_WIDTH_VAR                  "vlc_width"
-#define VOUT_HEIGHT_VAR                 "vlc_height"
 #define VOUT_WIDTH_DEFAULT              720
+#define VOUT_HEIGHT_VAR                 "vlc_height"
 #define VOUT_HEIGHT_DEFAULT             576
-
 #define VOUT_DEPTH_VAR                  "vlc_depth"
 #define VOUT_DEPTH_DEFAULT              15
 
-#define VOUT_FULLSCREEN_VAR             "vlc_fullscreen"
-#define VOUT_FULLSCREEN_DEFAULT         0
-
-#define VOUT_FULLSCREEN_DEPTH_VAR       "vlc_fullscreen_depth"
-#define VOUT_FULLSCREEN_DEPTH_DEFAULT   32
-
 /* Maximum width of a scaled source picture - this should be relatively high,
  * since higher stream values will result in no display at all. */
 #define VOUT_MAX_WIDTH                  4096
@@ -400,8 +408,14 @@
 /* Better be in advance when awakening than late... */
 #define VOUT_MWAIT_TOLERANCE            ((int)(0.020*CLOCK_FREQ))
 
+/* Time to sleep when waiting for a buffer (from vout or the video fifo).
+ * It should be approximately the time needed to perform a complete picture
+ * loop. Since it only happens when the video heap is full, it does not need
+ * to be too low, even if it blocks the decoder. */
+#define VOUT_OUTMEM_SLEEP               ((int)(0.020*CLOCK_FREQ))
+
 /* The default video output window title */
-#define VOUT_TITLE                      "VideoLAN Client 0.2.91"
+#define VOUT_TITLE                      "VideoLAN Client 0.2.92-pre1"
 
 /* Environment variable for framebuffer device, and default value */
 #define VOUT_FB_DEV_VAR                 "vlc_fb_dev"
@@ -416,12 +430,6 @@
 
 #define VPAR_IDLE_SLEEP                 ((int)(0.010*CLOCK_FREQ))
 
-/* Time to sleep when waiting for a buffer (from vout or the video fifo).
- * It should be approximately the time needed to perform a complete picture
- * loop. Since it only happens when the video heap is full, it does not need
- * to be too low, even if it blocks the decoder. */
-#define VPAR_OUTMEM_SLEEP               ((int)(0.020*CLOCK_FREQ))
-
 /* Optimization level, from 0 to 2 - 1 is generally a good compromise. Remember
  * that raising this level dramatically lengthens the compilation time. */
 #ifdef RELEASE
@@ -510,3 +518,4 @@
 #define PLAYLIST_LOOP_VAR               "vlc_playlist_loop"
 #define PLAYLIST_LOOP_DEFAULT           0
 
+#endif
