@@ -2,7 +2,7 @@
  * input_ext-dec.h: structures exported to the VideoLAN decoders
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: input_ext-dec.h,v 1.32 2001/05/01 12:22:18 sam Exp $
+ * $Id: input_ext-dec.h,v 1.41.2.2 2001/12/31 01:21:44 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Michel Kaempf <maxx@via.ecp.fr>
@@ -22,6 +22,17 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
+/* ES streams types - see ISO/IEC 13818-1 table 2-29 numbers */
+#define MPEG1_VIDEO_ES      0x01
+#define MPEG2_VIDEO_ES      0x02
+#define MPEG1_AUDIO_ES      0x03
+#define MPEG2_AUDIO_ES      0x04
+#define AC3_AUDIO_ES        0x81
+/* These ones might violate the norm : */
+#define DVD_SPU_ES          0x82
+#define LPCM_AUDIO_ES       0x83
+#define UNKNOWN_ES          0xFF
+
 /* Structures exported to the decoders */
 
 /*****************************************************************************
@@ -29,22 +40,25 @@
  *****************************************************************************
  * Describe a data packet.
  *****************************************************************************/
+#define DATA_PACKET                                                         \
+    /* start of the PS or TS packet */                                      \
+    byte_t *                p_demux_start;                                  \
+    /* start of the PES payload in this packet */                           \
+    byte_t *                p_payload_start;                                \
+    byte_t *                p_payload_end; /* guess ? :-) */                \
+    /* is the packet messed up ? */                                         \
+    boolean_t               b_discard_payload;
+
 typedef struct data_packet_s
 {
-    /* Nothing before this line, the code relies on that */
-    byte_t *                p_buffer;                     /* raw data packet */
-    long                    l_size;                           /* buffer size */
-
-    /* Decoders information */
-    byte_t *                p_payload_start;
-                                  /* start of the PES payload in this packet */
-    byte_t *                p_payload_end;                    /* guess ? :-) */
-    boolean_t               b_discard_payload;  /* is the packet messed up ? */
-
-    int *                   pi_refcount;
-
-    /* Used to chain the TS packets that carry data for a same PES or PSI */
+    /* Used to chain the packets that carry data for a same PES or PSI */
     struct data_packet_s *  p_next;
+
+    DATA_PACKET
+
+    /* Please note that at least one buffer allocator (in particular, the
+     * Next Generation Buffer Allocator) extends this structure with
+     * private data after DATA_PACKET. */
 } data_packet_t;
 
 /*****************************************************************************
@@ -55,6 +69,9 @@ typedef struct data_packet_s
  *****************************************************************************/
 typedef struct pes_packet_s
 {
+    /* Chained list to the next PES packet (depending on the context) */
+    struct pes_packet_s *   p_next;
+
     /* PES properties */
     boolean_t               b_data_alignment;  /* used to find the beginning of
                                                 * a video or audio unit      */
@@ -66,12 +83,15 @@ typedef struct pes_packet_s
     int                     i_rate;                /* current pace of reading
                                                     * (see stream_control.h) */
 
-    int                     i_pes_size;    /* size of the current PES packet */
+    unsigned int            i_pes_size;    /* size of the current PES packet */
 
-    /* Pointers to packets (packets are then linked by the p_prev and
-       p_next fields of the data_packet_t struct) */
+    /* Chained list to packets */
     data_packet_t *         p_first;      /* The first packet contained by this
                                            * PES (used by decoders). */
+    data_packet_t *         p_last;    /* The last packet contained by this
+                                          PES (used by the buffer allocator) */
+    unsigned int            i_nb_data; /* Number of data packets in the chained
+                                                                        list */
 } pes_packet_t;
 
 /*****************************************************************************
@@ -86,9 +106,9 @@ typedef struct decoder_fifo_s
     vlc_cond_t              data_wait;     /* fifo data conditional variable */
 
     /* Data */
-    pes_packet_t *          buffer[FIFO_SIZE + 1];
-    int                     i_start;
-    int                     i_end;
+    pes_packet_t *          p_first;
+    pes_packet_t **         pp_last;
+    int                     i_depth;   /* number of PES packets in the stack */
 
     /* Communication interface between input and decoders */
     boolean_t               b_die;          /* the decoder should return now */
@@ -99,18 +119,6 @@ typedef struct decoder_fifo_s
                                      /* function to use when releasing a PES */
 } decoder_fifo_t;
 
-/* Macros to manage a decoder_fifo_t structure. Please remember to take
- * data_lock before using them. */
-#define DECODER_FIFO_ISEMPTY( fifo )  ( (fifo).i_start == (fifo).i_end )
-#define DECODER_FIFO_ISFULL( fifo )   ( ( ((fifo).i_end + 1 - (fifo).i_start)\
-                                          & FIFO_SIZE ) == 0 )
-#define DECODER_FIFO_START( fifo )    ( (fifo).buffer[ (fifo).i_start ] )
-#define DECODER_FIFO_INCSTART( fifo ) ( (fifo).i_start = ((fifo).i_start + 1)\
-                                                         & FIFO_SIZE )
-#define DECODER_FIFO_END( fifo )      ( (fifo).buffer[ (fifo).i_end ] )
-#define DECODER_FIFO_INCEND( fifo )   ( (fifo).i_end = ((fifo).i_end + 1) \
-                                                       & FIFO_SIZE )
-
 /*****************************************************************************
  * bit_fifo_t : bit fifo descriptor
  *****************************************************************************
@@ -192,18 +200,22 @@ typedef struct bit_stream_s
 
 #if (WORD_TYPE == u32)
 #   define WORD_AT      U32_AT
+#   define WORD_SIGNED  s32
 #elif (WORD_TYPE == u64)
 #   define WORD_AT      U64_AT
+#   define WORD_SIGNED  s64
 #else
 #   error Unsupported WORD_TYPE
 #endif
 
 /*****************************************************************************
- * Protoypes from input_ext-dec.c
+ * Prototypes from input_ext-dec.c
  *****************************************************************************/
+#ifndef PLUGIN
 u32  UnalignedShowBits( struct bit_stream_s *, unsigned int );
 void UnalignedRemoveBits( struct bit_stream_s * );
 u32  UnalignedGetBits( struct bit_stream_s *, unsigned int );
+#endif
 
 /*****************************************************************************
  * AlignWord : fill in the bit buffer so that the byte pointer be aligned
@@ -252,7 +264,25 @@ static __inline__ u32 ShowBits( bit_stream_t * p_bit_stream,
                     >> (8 * sizeof(WORD_TYPE) - i_bits) );
     }
 
-    return UnalignedShowBits( p_bit_stream, i_bits );
+    return( UnalignedShowBits( p_bit_stream, i_bits ) );
+}
+
+/*****************************************************************************
+ * ShowSignedBits : return i_bits bits from the bit stream, using signed
+ *                  arithmetic
+ *****************************************************************************/
+static __inline__ s32 ShowSignedBits( bit_stream_t * p_bit_stream,
+                                      unsigned int i_bits )
+{
+    if( p_bit_stream->fifo.i_available >= i_bits )
+    {
+        return( (WORD_SIGNED)p_bit_stream->fifo.buffer
+                    >> (8 * sizeof(WORD_TYPE) - i_bits) );
+    }
+
+    /* You can probably do something a little faster, but now I'm tired. */
+    return( (WORD_SIGNED)(ShowBits( p_bit_stream, i_bits ) << (32 - i_bits))
+             >> (32 - i_bits) );
 }
 
 /*****************************************************************************
@@ -346,6 +376,30 @@ static __inline__ u32 GetBits( bit_stream_t * p_bit_stream,
     return UnalignedGetBits( p_bit_stream, i_bits );
 }
 
+/*****************************************************************************
+ * GetSignedBits : returns i_bits bits from the bit stream and removes them,
+ *                 using signed arithmetic
+ *                 XXX: do not use for 32 bits
+ *****************************************************************************/
+static __inline__ s32 GetSignedBits( bit_stream_t * p_bit_stream,
+                                     unsigned int i_bits )
+{
+    if( p_bit_stream->fifo.i_available >= i_bits )
+    {
+        s32             i_result;
+
+        p_bit_stream->fifo.i_available -= i_bits;
+        i_result = (WORD_SIGNED)p_bit_stream->fifo.buffer
+                        >> (8 * sizeof(WORD_TYPE) - i_bits);
+        p_bit_stream->fifo.buffer <<= i_bits;
+        return( i_result );
+    }
+
+    /* You can probably do something a little faster, but now I'm tired. */
+    return( (WORD_SIGNED)(GetBits( p_bit_stream, i_bits ) << (32 - i_bits))
+             >> (32 - i_bits) );
+}
+
 /*****************************************************************************
  * GetBits32 : returns 32 bits from the bit stream and removes them
  *****************************************************************************/
@@ -401,7 +455,7 @@ static __inline__ void RealignBits( bit_stream_t * p_bit_stream )
  * GetChunk : reads a large chunk of data
  *****************************************************************************
  * The position in the stream must be byte-aligned, if unsure call
- * RealignBits(). p_buffer must to a buffer at least as big as i_buf_len
+ * RealignBits(). p_buffer must point to a buffer at least as big as i_buf_len
  * otherwise your code will crash.
  *****************************************************************************/
 static __inline__ void GetChunk( bit_stream_t * p_bit_stream,
@@ -409,26 +463,27 @@ static __inline__ void GetChunk( bit_stream_t * p_bit_stream,
 {
     ptrdiff_t           i_available;
 
-    if( p_bit_stream->fifo.i_available )
+    /* We need to take care because i_buf_len may be < 4. */
+    while( p_bit_stream->fifo.i_available > 0 && i_buf_len )
     {
-        *((WORD_TYPE *)p_buffer) = WORD_AT( &p_bit_stream->fifo.buffer );
-        p_buffer += p_bit_stream->fifo.i_available >> 3;
-        i_buf_len -= p_bit_stream->fifo.i_available >> 3;
-        p_bit_stream->fifo.buffer = 0;
-        p_bit_stream->fifo.i_available = 0;
+        *p_buffer = p_bit_stream->fifo.buffer >> (8 * sizeof(WORD_TYPE) - 8);
+        p_buffer++;
+        i_buf_len--;
+        p_bit_stream->fifo.buffer <<= 8;
+        p_bit_stream->fifo.i_available -= 8;
     }
 
     if( (i_available = p_bit_stream->p_end - p_bit_stream->p_byte)
             >= i_buf_len )
     {
-        memcpy( p_buffer, p_bit_stream->p_byte, i_buf_len );
+        p_main->fast_memcpy( p_buffer, p_bit_stream->p_byte, i_buf_len );
         p_bit_stream->p_byte += i_buf_len;
     }
     else
     {
         do
         {
-            memcpy( p_buffer, p_bit_stream->p_byte, i_available );
+            p_main->fast_memcpy( p_buffer, p_bit_stream->p_byte, i_available );
             p_bit_stream->p_byte = p_bit_stream->p_end;
             p_buffer += i_available;
             i_buf_len -= i_available;
@@ -439,7 +494,7 @@ static __inline__ void GetChunk( bit_stream_t * p_bit_stream,
 
         if( i_buf_len )
         {
-            memcpy( p_buffer, p_bit_stream->p_byte, i_buf_len );
+            p_main->fast_memcpy( p_buffer, p_bit_stream->p_byte, i_buf_len );
             p_bit_stream->p_byte += i_buf_len;
         }
     }
@@ -451,38 +506,6 @@ static __inline__ void GetChunk( bit_stream_t * p_bit_stream,
 }
 
 
-/*
- * The following functions are now deprecated.
- */
-
-static __inline__ byte_t _GetByte( bit_stream_t * p_bit_stream )
-{
-    if ( p_bit_stream->p_byte >= p_bit_stream->p_end )
-    {
-        p_bit_stream->pf_next_data_packet( p_bit_stream );
-    }
-
-    return( *(p_bit_stream->p_byte++) );
-}
-
-static __inline__ void NeedBits( bit_stream_t * p_bit_stream, int i_bits )
-{
-    while ( p_bit_stream->fifo.i_available < i_bits )
-    {
-        p_bit_stream->fifo.buffer |= ((WORD_TYPE)_GetByte( p_bit_stream ))
-                                     << (8 * sizeof(WORD_TYPE) - 8
-                                            - p_bit_stream->fifo.i_available);
-        p_bit_stream->fifo.i_available += 8;
-    }
-}
-
-static __inline__ void DumpBits( bit_stream_t * p_bit_stream, int i_bits )
-{
-    p_bit_stream->fifo.buffer <<= i_bits;
-    p_bit_stream->fifo.i_available -= i_bits;
-}
-
-
 /*
  * Communication interface between input and decoders
  */
@@ -507,71 +530,11 @@ typedef struct decoder_config_s
 } decoder_config_t;
 
 /*****************************************************************************
- * vdec_config_t
- *****************************************************************************
- * Pointers given to video decoders threads.
+ * Prototypes from input_dec.c
  *****************************************************************************/
-struct vout_thread_s;
+#ifndef PLUGIN
+void DecoderError      ( struct decoder_fifo_s * p_fifo );
+#else
+#   define DecoderError p_symbols->DecoderError
+#endif
 
-typedef struct vdec_config_s
-{
-    struct picture_s *   (* pf_create_picture)( struct vout_thread_s *,
-                                                int i_type, int i_width,
-                                                int i_height );
-    void                 (* pf_destroy_picture)( struct vout_thread_s *,
-                                                 struct picture_s * );
-    void                 (* pf_display_picture)( struct vout_thread_s *,
-                                                 struct picture_s * );
-    void                 (* pf_date_picture)( struct vout_thread_s *,
-                                              struct picture_s *, mtime_t date );
-    void                 (* pf_link_picture)( struct vout_thread_s *,
-                                              struct picture_s *, mtime_t date );
-    void                 (* pf_unlink_picture)( struct vout_thread_s *,
-                                                struct picture_s *, mtime_t date );
-    struct subpicture_s *(* pf_create_subpicture)( struct vout_thread_s *,
-                                                   int i_type, int i_size );
-    void                 (* pf_destroy_subpicture)( struct vout_thread_s *,
-                                                    struct subpicture_s * );
-    void                 (* pf_display_subpicture)( struct vout_thread_s *,
-                                                    struct subpicture_s * );
-
-    decoder_config_t        decoder_config;
-} vdec_config_t;
-
-/*****************************************************************************
- * adec_config_t
- *****************************************************************************
- * Pointers given to audio decoders threads.
- *****************************************************************************/
-typedef struct adec_config_s
-{
-    struct aout_fifo_s * (* pf_create_fifo)( struct aout_fifo_s * );
-    void                 (* pf_destroy_fifo)( void );
-
-    decoder_config_t        decoder_config;
-} adec_config_t;
-
-
-/*
- * Communication interface between decoders and input
- */
-
-/*****************************************************************************
- * decoder_capabilities_t
- *****************************************************************************
- * Structure returned by a call to GetCapabilities() of the decoder.
- *****************************************************************************/
-typedef struct decoder_capabilities_s
-{
-    int                     i_dec_type;
-    u8                      i_stream_type;   /* == i_type in es_descriptor_t */
-    int                     i_weight; /* for a given stream type, the decoder
-                                       * with higher weight will be spawned  */
-
-    vlc_thread_t         (* pf_create_thread)( void * );
-} decoder_capabilities_t;
-
-/* Decoder types */
-#define NONE_D              0
-#define VIDEO_D             1
-#define AUDIO_D             2
