@@ -4,7 +4,7 @@
  * control the pace of reading. 
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: input_ext-intf.h,v 1.38 2001/05/30 17:03:11 sam Exp $
+ * $Id: input_ext-intf.h,v 1.51.2.1 2001/12/31 01:21:44 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -26,7 +26,6 @@
 /*
  * Communication input -> interface
  */
-#define INPUT_MAX_PLUGINS   1
 /* FIXME ! */
 #define REQUESTED_MPEG         1
 #define REQUESTED_AC3          2
@@ -66,18 +65,19 @@ typedef struct es_descriptor_s
 
     /* PES parser information */
     struct pes_packet_s *   p_pes;                            /* Current PES */
-    struct data_packet_s *  p_last;   /* The last packet gathered at present */
     int                     i_pes_real_size;   /* as indicated by the header */
 
     /* Decoder information */
     struct decoder_fifo_s * p_decoder_fifo;
     vlc_thread_t            thread_id;                  /* ID of the decoder */
 
-#ifdef STATS
-    count_t                 c_payload_bytes;/* total of payload useful bytes */
     count_t                 c_packets;                 /* total packets read */
     count_t                 c_invalid_packets;       /* invalid packets read */
-#endif
+
+    /* Module properties */
+    struct module_s *         p_module;
+    struct decoder_config_s * p_config;
+
 } es_descriptor_t;
 
 /* Special PID values - note that the PID is only on 13 bits, and that values
@@ -86,16 +86,6 @@ typedef struct es_descriptor_s
 #define CONDITIONNAL_ACCESS_TABLE_PID   0x0001                   /* not used */
 #define EMPTY_ID                        0xffff    /* empty record in a table */
  
-/* ES streams types - see ISO/IEC 13818-1 table 2-29 numbers */
-#define MPEG1_VIDEO_ES      0x01
-#define MPEG2_VIDEO_ES      0x02
-#define MPEG1_AUDIO_ES      0x03
-#define MPEG2_AUDIO_ES      0x04
-#define AC3_AUDIO_ES        0x81
-/* These ones might violate the norm : */
-#define DVD_SPU_ES          0x82
-#define LPCM_AUDIO_ES       0x83
-#define UNKNOWN_ES          0xFF
 
 /* ES Categories to be used by interface plugins */
 #define VIDEO_ES        0x00
@@ -103,6 +93,7 @@ typedef struct es_descriptor_s
 #define SPU_ES          0x02
 #define NAV_ES          0x03
 #define UNKNOWN_ES      0xFF
+
 /*****************************************************************************
  * pgrm_descriptor_t
  *****************************************************************************
@@ -125,6 +116,7 @@ typedef struct pgrm_descriptor_s
     mtime_t                 cr_ref, sysdate_ref;
     mtime_t                 last_cr; /* reference to detect unexpected stream
                                       * discontinuities                      */
+    mtime_t                 last_syscr;
     count_t                 c_average_count;
                            /* counter used to compute dynamic average values */
     int                     i_synchro_state;
@@ -204,6 +196,7 @@ typedef struct stream_descriptor_s
 
     /* New status and rate requested by the interface */
     int                     i_new_status, i_new_rate;
+    int                     b_new_mute;          /* int because it can be -1 */
     vlc_cond_t              stream_wait; /* interface -> input in case of a
                                           * status change request            */
 
@@ -213,7 +206,9 @@ typedef struct stream_descriptor_s
     /* Programs descriptions */
     int                     i_pgrm_number;    /* size of the following array */
     pgrm_descriptor_t **    pp_programs;        /* array of pointers to pgrm */
-
+    pgrm_descriptor_t *     p_selected_program;   /* currently 
+                                                 selected program */
+    pgrm_descriptor_t *     p_new_program;        /* Newly selected program */
     /* ES descriptions */
     int                     i_es_number;
     es_descriptor_t **      pp_es;             /* carried elementary streams */
@@ -226,27 +221,13 @@ typedef struct stream_descriptor_s
 
     /* Stream control */
     stream_ctrl_t           control;
+
+    /* Statistics */
+    count_t                 c_packets_read;                  /* packets read */
+    count_t                 c_packets_trashed;            /* trashed packets */
 } stream_descriptor_t;
 
-/*****************************************************************************
- * i_p_config_t
- *****************************************************************************
- * This structure gives plugins pointers to the useful functions of input
- *****************************************************************************/
-struct input_thread_s;
-struct data_packet_s;
-struct es_descriptor_s;
-
-typedef struct i_p_config_s
-{
-    int                 (* pf_peek_stream)( struct input_thread_s *,
-                                            byte_t * buffer, size_t );
-    void                (* pf_demux_pes)( struct input_thread_s *,
-                                          struct data_packet_s *,
-                                          struct es_descriptor_s *,
-                                          boolean_t b_unit_start,
-                                          boolean_t b_packet_lost );
-} i_p_config_t;
+#define MUTE_NO_CHANGE      -1
 
 /*****************************************************************************
  * input_thread_t
@@ -254,6 +235,7 @@ typedef struct i_p_config_s
  * This structure includes all the local static variables of an input thread
  *****************************************************************************/
 struct vout_thread_s;
+struct bit_stream_s;
 
 typedef struct input_thread_s
 {
@@ -272,10 +254,15 @@ typedef struct input_thread_s
     void                 (* pf_open)( struct input_thread_s * );
     void                 (* pf_close)( struct input_thread_s * );
     void                 (* pf_end)( struct input_thread_s * );
+    void                 (* pf_init_bit_stream)( struct bit_stream_s *,
+                              struct decoder_fifo_s *,
+                void (* pf_bitstream_callback)( struct bit_stream_s *,
+                                                boolean_t ),
+                              void * );
 
     /* Read & Demultiplex */
     int                  (* pf_read)( struct input_thread_s *,
-                                      struct data_packet_s * pp_packets[] );
+                                      struct data_packet_s ** );
     void                 (* pf_demux)( struct input_thread_s *,
                                        struct data_packet_s * );
 
@@ -287,6 +274,8 @@ typedef struct input_thread_s
     void                 (* pf_delete_pes)( void *, struct pes_packet_s * );
 
     /* Stream control capabilities */
+    int                  (* pf_set_program)( struct input_thread_s *,
+                                             struct pgrm_descriptor_s * );
     int                  (* pf_set_area)( struct input_thread_s *,
                                           struct input_area_s * );
     int                  (* pf_rewind)( struct input_thread_s * );
@@ -294,33 +283,18 @@ typedef struct input_thread_s
                                             * backwards (it's gonna be fun)  */
     void                 (* pf_seek)( struct input_thread_s *, off_t );
 
-    /* Special callback functions */
-    void                 (* pf_file_open )     ( struct input_thread_s * );
-    void                 (* pf_file_close )    ( struct input_thread_s * );
-#if !defined( SYS_BEOS ) && !defined( SYS_NTO )
-    void                 (* pf_network_open )  ( struct input_thread_s * );
-    void                 (* pf_network_close ) ( struct input_thread_s * );
-#endif
-
-    i_p_config_t            i_p_config;              /* plugin configuration */
     char *                  p_source;
 
     int                     i_handle;           /* socket or file descriptor */
-    int                     i_read_once;        /* number of packet read by
-                                                 * pf_read once */
+    FILE *                  p_stream;                       /* if applicable */
+    void *                  p_handle;          /* if i_handle isn't suitable */
     void *                  p_method_data;     /* data of the packet manager */
     void *                  p_plugin_data;             /* data of the plugin */
 
     /* General stream description */
     stream_descriptor_t     stream;                            /* PAT tables */
 
-#ifdef STATS
     count_t                 c_loops;
-    count_t                 c_bytes;                           /* bytes read */
-    count_t                 c_payload_bytes;         /* payload useful bytes */
-    count_t                 c_packets_read;                  /* packets read */
-    count_t                 c_packets_trashed;            /* trashed packets */
-#endif
 } input_thread_t;
 
 /* Input methods */
@@ -348,12 +322,12 @@ typedef struct input_thread_s
 /*****************************************************************************
  * Prototypes
  *****************************************************************************/
+#ifndef PLUGIN
 struct input_thread_s * input_CreateThread ( struct playlist_item_s *,
                                              int *pi_status );
 void input_DestroyThread( struct input_thread_s *, int *pi_status );
 
 void input_SetStatus( struct input_thread_s *, int );
-void input_SetRate  ( struct input_thread_s *, int );
 void input_Seek     ( struct input_thread_s *, off_t );
 void input_DumpStream( struct input_thread_s * );
 char * input_OffsetToTime( struct input_thread_s *, char * psz_buffer, off_t );
@@ -361,5 +335,8 @@ int  input_ChangeES ( struct input_thread_s *, struct es_descriptor_s *, u8 );
 int  input_ToggleES ( struct input_thread_s *,
                       struct es_descriptor_s *,
                       boolean_t );
-int  input_ChangeArea( input_thread_t *, input_area_t * );
-
+int  input_ChangeArea( struct input_thread_s *, struct input_area_s * );
+int  input_ToggleGrayscale( struct input_thread_s * );
+int  input_ToggleMute( struct input_thread_s * );
+int  input_SetSMP( struct input_thread_s *, int );
+#endif
