@@ -1,8 +1,8 @@
 /*****************************************************************************
  * aout_directx.c: Windows DirectX audio output method
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
- * $Id: aout_directx.c,v 1.11 2001/11/12 11:13:18 massiot Exp $
+ * Copyright (C) 2001 VideoLAN
+ * $Id: aout_directx.c,v 1.13 2001/12/07 18:33:07 sam Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -37,8 +37,8 @@
 #include <stdio.h>                                           /* "intf_msg.h" */
 #include <stdlib.h>                            /* calloc(), malloc(), free() */
 
-#include "config.h"
 #include "common.h"                                     /* boolean_t, byte_t */
+#include "intf_msg.h"                        /* intf_DbgMsg(), intf_ErrMsg() */
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
@@ -51,9 +51,6 @@
 
 #include "audio_output.h"                                   /* aout_thread_t */
 
-#include "intf_msg.h"                        /* intf_DbgMsg(), intf_ErrMsg() */
-#include "main.h"
-
 #include "modules.h"
 #include "modules_export.h"
 
