@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vout_events.c: Windows DirectX video output events handler
  *****************************************************************************
- * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: vout_events.c,v 1.3 2001/07/25 08:41:21 gbazin Exp $
+ * Copyright (C) 2001 VideoLAN
+ * $Id: vout_events.c,v 1.6 2001/12/07 18:33:07 sam Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -32,8 +32,8 @@
 #include <stdlib.h>                                                /* free() */
 #include <string.h>                                            /* strerror() */
 
-#include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
@@ -44,11 +44,14 @@
 
 #include <windows.h>
 #include <windowsx.h>
-#include <directx.h>
 
-#include "intf_msg.h"
+#if defined( _MSC_VER )
+#   include <ddraw.h>
+#else
+#   include <directx.h>
+#endif
+
 #include "interface.h"
-#include "main.h"
 
 #include "modules.h"
 #include "modules_export.h"
