@@ -2,7 +2,7 @@
  * aout_dummy.c : dummy audio output plugin
  *****************************************************************************
  * Copyright (C) 2000, 2001 VideoLAN
- * $Id: aout_dummy.c,v 1.14 2001/07/12 20:44:52 reno Exp $
+ * $Id: aout_dummy.c,v 1.16 2001/12/07 18:33:07 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -29,16 +29,16 @@
  *****************************************************************************/
 #include "defs.h"
 
-#include "config.h"
+#include <string.h>
+
 #include "common.h"                                     /* boolean_t, byte_t */
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
 
 #include "audio_output.h"                                   /* aout_thread_t */
 
-#include "main.h"
-
 #include "modules.h"
 #include "modules_export.h"
 
