@@ -1,8 +1,8 @@
 /*****************************************************************************
  * idctmmxext.c : MMX EXT IDCT module
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
- * $Id: idctmmxext.c,v 1.10 2001/06/03 12:47:21 sam Exp $
+ * Copyright (C) 1999-2001 VideoLAN
+ * $Id: idctmmxext.c,v 1.17 2001/11/28 15:08:05 massiot Exp $
  *
  * Authors: Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
  *          Michel Lespinasse <walken@zoy.org>
@@ -26,6 +26,7 @@
  *****************************************************************************/
 
 #define MODULE_NAME idctmmxext
+#include "modules_inner.h"
 
 /*****************************************************************************
  * Preamble
@@ -33,35 +34,27 @@
 #include "defs.h"
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"                                              /* TestCPU() */
 
-#include "video.h"
-#include "video_output.h"
-
-#include "video_decoder.h"
-
-#include "modules.h"
-#include "modules_inner.h"
-
-#include "vdec_block.h"
-#include "vdec_idct.h"
-
-#include "attributes.h"
 #include "mmx.h"
 
+#include "idct.h"
+#include "block_mmx.h"
+
+#include "modules.h"
 #include "modules_export.h"
 
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
 static void idct_getfunctions( function_list_t * p_function_list );
-static int  idct_Probe      ( probedata_t *p_data );
-static void vdec_NormScan   ( u8 ppi_scan[2][64] );
 
 /*****************************************************************************
  * Build configuration tree.
@@ -86,23 +79,6 @@ MODULE_DEACTIVATE_STOP
 
 /* Following functions are local */
 
-/*****************************************************************************
- * Functions exported as capabilities.
- *****************************************************************************/
-static void idct_getfunctions( function_list_t * p_function_list )
-{
-    p_function_list->pf_probe = idct_Probe;
-#define F p_function_list->functions.idct
-    F.pf_idct_init = _M( vdec_InitIDCT );
-    F.pf_sparse_idct = _M( vdec_SparseIDCT );
-    F.pf_idct = _M( vdec_IDCT );
-    F.pf_norm_scan = vdec_NormScan;
-    F.pf_decode_init = _M( vdec_InitDecode );
-    F.pf_decode_mb_c = _M( vdec_DecodeMacroblockC );
-    F.pf_decode_mb_bw = _M( vdec_DecodeMacroblockBW );
-#undef F
-}
-
 /*****************************************************************************
  * idct_Probe: return a preference score
  *****************************************************************************/
@@ -113,7 +89,8 @@ static int idct_Probe( probedata_t *p_data )
         return( 0 );
     }
 
-    if( TestMethod( IDCT_METHOD_VAR, "idctmmxext" ) )
+    if( TestMethod( IDCT_METHOD_VAR, "idctmmxext" )
+         || TestMethod( IDCT_METHOD_VAR, "mmxext" ) )
     {
         return( 999 );
     }
@@ -123,9 +100,9 @@ static int idct_Probe( probedata_t *p_data )
 }
 
 /*****************************************************************************
- * vdec_NormScan : This IDCT uses reordered coeffs, so we patch the scan table
+ * NormScan : This IDCT uses reordered coeffs, so we patch the scan table
  *****************************************************************************/
-static void vdec_NormScan( u8 ppi_scan[2][64] )
+static void NormScan( u8 ppi_scan[2][64] )
 {
     int     i, j;
 
@@ -140,7 +117,7 @@ static void vdec_NormScan( u8 ppi_scan[2][64] )
 }
 
 /*****************************************************************************
- * vdec_IDCT :
+ * IDCT :
  *****************************************************************************/
 #define ROW_SHIFT 11
 #define COL_SHIFT 6
@@ -417,8 +394,7 @@ static s32 rounder3[] ATTR_ALIGN(8) =
 static s32 rounder5[] ATTR_ALIGN(8) =
     rounder (-0.441341716183);  // C3*(-C5/C4+C5-C3)/2
 
-void _M( vdec_IDCT )( vdec_thread_t * p_vdec, dctelem_t * p_block,
-                      int i_idontcare )
+static void IDCT( dctelem_t * p_block )
 {
     static dctelem_t table04[] ATTR_ALIGN(16) =
         table (22725, 21407, 19266, 16384, 12873,  8867, 4520);
@@ -451,3 +427,5 @@ void _M( vdec_IDCT )( vdec_thread_t * p_vdec, dctelem_t * p_block,
     Col( p_block, 4 );
 }
 
+#include "idct_sparse.h"
+#include "idct_decl.h"
