@@ -1,4 +1,3 @@
-
 /*****************************************************************************
  * vout_macosx.c: MacOS X video output plugin
  *****************************************************************************
@@ -34,20 +33,18 @@
 #include <stdlib.h>                                                /* free() */
 #include <string.h>                                            /* strerror() */
 
-#include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
 
 #include "interface.h"
-#include "intf_msg.h"
 
 #include "video.h"
 #include "video_output.h"
 
 #include "modules.h"
-#include "main.h"
 
 #include "macosx.h"
 
@@ -154,9 +151,7 @@ static int vout_Init( vout_thread_t *p_vout )
     p_vout->b_need_render = 0 ;
     p_vout->i_bytes_per_line = p_vout->i_width ;
     p_vout->p_sys->c_codec = 'NONE' ;
-    vlc_mutex_lock( &p_vout->p_sys->osx_communication.lock ) ;
-        p_vout->p_sys->osx_communication.i_changes |= OSX_VOUT_INTF_REQUEST_QDPORT ;
-    vlc_mutex_unlock( &p_vout->p_sys->osx_communication.lock ) ;
+    p_vout->p_sys->osx_communication.i_changes |= OSX_VOUT_INTF_REQUEST_QDPORT ;
     
     return 0 ;
 }
@@ -166,11 +161,8 @@ static int vout_Init( vout_thread_t *p_vout )
  *****************************************************************************/
 static void vout_End( vout_thread_t *p_vout )
 {
-    vlc_mutex_lock( &p_vout->p_sys->osx_communication.lock ) ;
-        p_vout->p_sys->osx_communication.i_changes |= OSX_VOUT_INTF_RELEASE_QDPORT ;
-    vlc_mutex_unlock( &p_vout->p_sys->osx_communication.lock ) ;
-    
     dispose_QTSequence( p_vout ) ;
+    p_vout->p_sys->osx_communication.i_changes |= OSX_VOUT_INTF_RELEASE_QDPORT ;
 }
 
 /*****************************************************************************
@@ -191,22 +183,22 @@ static void vout_Destroy( vout_thread_t *p_vout )
  *****************************************************************************/
 static int vout_Manage( vout_thread_t *p_vout )
 {    
-    vlc_mutex_lock( &p_vout->p_sys->osx_communication.lock ) ;
-        if ( p_vout->p_sys->osx_communication.i_changes & OSX_INTF_VOUT_QDPORT_CHANGE ) {
-            dispose_QTSequence( p_vout ) ;
-            create_QTSequenceBestCodec( p_vout ) ;
-        }
-        else if ( p_vout->p_sys->osx_communication.i_changes & OSX_INTF_VOUT_SIZE_CHANGE ) {
-            fillout_ScalingMatrix( p_vout ) ;
-            SetDSequenceMatrix( p_vout->p_sys->i_seq, p_vout->p_sys->p_matrix ) ;
-        }
-        
-        p_vout->p_sys->osx_communication.i_changes &= ~( 
-            OSX_INTF_VOUT_QDPORT_CHANGE |
-            OSX_INTF_VOUT_SIZE_CHANGE
-        ) ;
-    vlc_mutex_unlock( &p_vout->p_sys->osx_communication.lock ) ;
-        
+      if ( p_vout->p_sys->osx_communication.i_changes & OSX_INTF_VOUT_QDPORT_CHANGE ) {
+          dispose_QTSequence( p_vout ) ;
+          create_QTSequenceBestCodec( p_vout ) ;
+      }
+      else if ( p_vout->p_sys->osx_communication.i_changes & OSX_INTF_VOUT_SIZE_CHANGE ) {
+          if ( p_vout->p_sys->c_codec != 'NONE' ) {
+              fillout_ScalingMatrix( p_vout ) ;
+              SetDSequenceMatrix( p_vout->p_sys->i_seq, p_vout->p_sys->p_matrix ) ;
+          }
+      }
+
+      p_vout->p_sys->osx_communication.i_changes &= ~( 
+          OSX_INTF_VOUT_QDPORT_CHANGE |
+          OSX_INTF_VOUT_SIZE_CHANGE
+      ) ;
+
     return 0 ;
 }
 
@@ -220,7 +212,7 @@ static int vout_Manage( vout_thread_t *p_vout )
 void vout_Display( vout_thread_t *p_vout )
 {
     CodecFlags out_flags ;
-    
+
     switch (p_vout->p_sys->c_codec)
     {
         case 'yuv2':
@@ -245,6 +237,8 @@ void vout_Display( vout_thread_t *p_vout )
  		nil
             ) ;            
             break ;
+       default:
+           intf_WarnMsg( 1, "vout_macosx: vout_Display called, but no codec available" ) ;
     }
 }
 
@@ -364,6 +358,9 @@ static int create_QTSequenceBestCodec( vout_thread_t *p_vout )
 
 static void dispose_QTSequence( vout_thread_t *p_vout )
 {
+    if (p_vout->p_sys->c_codec == 'NONE')
+    	return ;
+    	
     CDSequenceEnd( p_vout->p_sys->i_seq ) ;
     switch (p_vout->p_sys->c_codec)
     {
@@ -371,8 +368,8 @@ static void dispose_QTSequence( vout_thread_t *p_vout )
             free( (void *)p_vout->p_sys->p_yuv2 ) ;
             p_vout->p_sys->i_yuv2_size = 0 ;
             break ;
-        case 'y420': 
-            break ;
+        default:
+	    break ;            
     }
     p_vout->p_sys->c_codec = 'NONE' ;
 }
