@@ -2,9 +2,10 @@
  * input_ps.h: thread structure of the PS plugin
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: input_ps.h,v 1.1 2001/02/08 04:43:27 sam Exp $
+ * $Id: input_ps.h,v 1.9.2.1 2001/12/31 01:21:45 massiot Exp $
  *
- * Authors:
+ * Authors: Christophe Massiot <massiot@via.ecp.fr>
+ *          Cyril Deguet <asmax@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +22,5 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-/*****************************************************************************
- * thread_ps_data_t: extension of input_thread_t
- *****************************************************************************/
-typedef struct thread_ps_data_s
-{
-    /* We're necessarily reading a file. */
-    FILE *                  stream;
-} thread_ps_data_t;
+#define PS_READ_ONCE 50
+
