@@ -2,7 +2,7 @@
  * intf_qt.cpp: Qt interface
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: intf_qt.cpp,v 1.4 2001/04/16 07:40:11 sam Exp $
+ * $Id: intf_qt.cpp,v 1.9 2001/12/07 18:33:08 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -37,21 +37,20 @@ extern "C"
 #include <string.h>                                            /* strerror() */
 #include <stdio.h>
 
-#include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
-#include "modules.h"
 
 #include "stream_control.h"
 #include "input_ext-intf.h"
 
-#include "intf_msg.h"
 #include "intf_playlist.h"
 #include "interface.h"
 
-#include "main.h"
+#include "modules.h"
+#include "modules_export.h"
 
 } /* extern "C" */
 
@@ -127,7 +126,7 @@ private slots:
     void DateDisplay  ( int );
     void About ( void );
 
-    void Unimplemented( void ) { intf_WarnMsg( 3, "intf warning: "
+    void Unimplemented( void ) { intf_WarnMsg( 1, "intf warning: "
                                  "unimplemented function" ); };
 
 private:
@@ -465,7 +464,7 @@ void IntfWindow::About( void )
 void IntfWindow::Manage( void )
 {
     /* Manage the slider */
-    if( p_intf->p_input != NULL )
+    if( p_intf->p_input != NULL && p_intf->p_input->stream.b_seekable )
     {
         int i_value = p_slider->value();
 
