@@ -3,9 +3,7 @@
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
  *
- * Author: Johan Bilien <jobi@via.ecp.fr>, based on Stephane Borel's DVD input
- *                                         and on Christophe Massiot and
- *                                         Cyril Deguet's PS input
+ * Author: Johan Bilien <jobi@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,9 +50,8 @@
 #   include <sys/uio.h>                                      /* struct iovec */
 #endif
 
-
-#include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
@@ -63,10 +60,6 @@
 #   include "input_iovec.h"
 #endif
 
-#include "intf_msg.h"
-
-#include "main.h"
-
 #include "stream_control.h"
 #include "input_ext-intf.h"
 #include "input_ext-dec.h"
@@ -92,19 +85,20 @@
  * Local prototypes
  *****************************************************************************/
 /* called from outside */
-static int  VCDProbe     ( probedata_t *p_data );
-static void VCDInit      ( struct input_thread_s * );
-static int  VCDRead      ( struct input_thread_s *, data_packet_t ** );
-static int  VCDSetArea   ( struct input_thread_s *, struct input_area_s * ); 
-static void VCDOpen      ( struct input_thread_s *);
-static void VCDClose     ( struct input_thread_s *);
-static void VCDEnd       ( struct input_thread_s *);
-static void VCDSeek      ( struct input_thread_s *, off_t );
-static int  VCDRewind    ( struct input_thread_s * );
+static int  VCDProbe        ( probedata_t *p_data );
+static void VCDInit         ( struct input_thread_s * );
+static int  VCDRead         ( struct input_thread_s *, data_packet_t ** );
+static int  VCDSetArea      ( struct input_thread_s *, struct input_area_s * ); 
+static int  VCDSetProgram   ( struct input_thread_s *, pgrm_descriptor_t * ); 
+static void VCDOpen         ( struct input_thread_s *);
+static void VCDClose         ( struct input_thread_s *);
+static void VCDEnd          ( struct input_thread_s *);
+static void VCDSeek         ( struct input_thread_s *, off_t );
+static int  VCDRewind       ( struct input_thread_s * );
 static struct data_packet_s * NewPacket( void *, size_t );
 static pes_packet_t *         NewPES   ( void * );
-static void DeletePacket ( void *, data_packet_t * );
-static void DeletePES    ( void *, pes_packet_t *);
+static void DeletePacket     ( void *, data_packet_t * );
+static void DeletePES       ( void *, pes_packet_t *);
 
 
 
@@ -115,21 +109,22 @@ static void DeletePES    ( void *, pes_packet_t *);
 void _M( input_getfunctions )( function_list_t * p_function_list )
 {
 #define input p_function_list->functions.input
-    p_function_list->pf_probe = VCDProbe ;
-    input.pf_init             = VCDInit ;
-    input.pf_open             = VCDOpen ;
-    input.pf_close            = VCDClose ;
-    input.pf_end              = VCDEnd ;
+    p_function_list->pf_probe = VCDProbe;
+    input.pf_init             = VCDInit;
+    input.pf_open             = VCDOpen;
+    input.pf_close            = VCDClose;
+    input.pf_end              = VCDEnd;
     input.pf_init_bit_stream  = InitBitstream;
-    input.pf_read             = VCDRead ;
-    input.pf_set_area         = VCDSetArea ;
+    input.pf_read             = VCDRead;
+    input.pf_set_area         = VCDSetArea;
+    input.pf_set_program      = VCDSetProgram;
     input.pf_demux            = input_DemuxPS;
     input.pf_new_packet       = NewPacket;
     input.pf_new_pes          = NewPES;
     input.pf_delete_packet    = DeletePacket;
     input.pf_delete_pes       = DeletePES;
-    input.pf_rewind           = VCDRewind ;
-    input.pf_seek             = VCDSeek ;
+    input.pf_rewind           = VCDRewind;
+    input.pf_seek             = VCDSeek;
 #undef input
 }
 
@@ -324,8 +319,8 @@ static void VCDInit( input_thread_t * p_input )
         area[i]->i_id = i;
 
         /* Absolute start offset and size */
-        area[i]->i_start = p_vcd->tracks_sector[i+1];
-        area[i]->i_size = p_vcd->tracks_sector[i+2] - p_vcd->tracks_sector[i+1];
+        area[i]->i_start = p_vcd->tracks_sector[i];
+        area[i]->i_size = p_vcd->tracks_sector[i+1] - p_vcd->tracks_sector[i];
 
         /* Number of chapters */
         area[i]->i_part_nb = 0;   // will be the entry points
@@ -335,7 +330,7 @@ static void VCDInit( input_thread_t * p_input )
         area[i]->i_angle_nb = 1; // no angle support in VCDs
         area[i]->i_angle = 1;
 
-        area[i]->i_plugin_data = p_vcd->tracks_sector[i+1];
+        area[i]->i_plugin_data = p_vcd->tracks_sector[i];
     }   
 #undef area
 
@@ -343,7 +338,7 @@ static void VCDInit( input_thread_t * p_input )
     i_title = main_GetIntVariable( INPUT_TITLE_VAR, 1 );
     if( i_title <= 0)
     {
-        i_title = 2;
+        i_title = 1;
     }
     
     // p_vcd->current_track = i_title-1 ;
@@ -364,12 +359,13 @@ static void VCDInit( input_thread_t * p_input )
 
     /* Set program information. */
 
-    input_AddProgram( p_input, 0, sizeof( stream_ps_data_t ) ); 
+    input_AddProgram( p_input, 0, sizeof( stream_ps_data_t ) );
+    p_input->stream.p_selected_program = p_input->stream.pp_programs[0];
 
     /* No PSM to read in disc mode, we already have all information */
-    p_input->stream.pp_programs[0]->b_is_ok = 1;
+    p_input->stream.p_selected_program->b_is_ok = 1;
 
-    p_es = input_AddES( p_input, p_input->stream.pp_programs[0], 0xe0, 0 );
+    p_es = input_AddES( p_input, p_input->stream.p_selected_program, 0xe0, 0 );
     p_es->i_stream_id = 0xe0;
     p_es->i_type = MPEG1_VIDEO_ES;
     p_es->i_cat = VIDEO_ES;
@@ -380,7 +376,7 @@ static void VCDInit( input_thread_t * p_input )
     }
     
     p_es = input_AddES( p_input,
-                p_input->stream.pp_programs[0], 0xc0, 0 );
+                p_input->stream.p_selected_program, 0xc0, 0 );
     p_es->i_stream_id = 0xc0;
     p_es->i_type = MPEG1_AUDIO_ES;
     p_es->b_audio = 1;
@@ -411,6 +407,16 @@ static void VCDEnd( input_thread_t * p_input )
 
 }
 
+/*****************************************************************************
+ * VCDSetProgram: Does nothing since a VCD is mono_program
+ *****************************************************************************/
+static int VCDSetProgram( input_thread_t * p_input, 
+            pgrm_descriptor_t * p_program)
+{
+    return 0;
+}
+
+
 /*****************************************************************************
  * VCDSetArea: initialize input data for title x, chapter y.
  * It should be called for each user navigation request.
@@ -428,14 +434,14 @@ static int VCDSetArea( input_thread_t * p_input, input_area_t * p_area )
     {
         /* Reset the Chapter position of the old title */
         p_input->stream.p_selected_area->i_part = 0;
+        p_input->stream.p_selected_area->i_tell = 0;
         
         /* Change the default area */
-        p_input->stream.p_selected_area =
-                    p_input->stream.pp_areas[p_area->i_id];
+        p_input->stream.p_selected_area = p_area;
 
         /* Change the current track */
         /* The first track is not a valid one  */
-        p_vcd->current_track = p_area->i_id + 1 ;
+        p_vcd->current_track = p_area->i_id ;
         p_vcd->current_sector = p_vcd->tracks_sector[p_vcd->current_track] ;
     }
     /* warn interface that something has changed */
@@ -588,8 +594,8 @@ static int VCDRead( input_thread_t * p_input,
     
             if( U32_AT(p_buffer) != 0x1B9 )
             {
-                memcpy( p_data->p_buffer, p_buffer + i_index,
-                        6 + i_packet_size );
+                p_main->fast_memcpy( p_data->p_buffer, p_buffer + i_index,
+                                     6 + i_packet_size );
                 i_index += ( 6 + i_packet_size );
     
             }
@@ -610,9 +616,6 @@ static int VCDRead( input_thread_t * p_input,
     }
 
 
-
-
-
     
     vlc_mutex_lock( &p_input->stream.stream_lock );
 
@@ -629,8 +632,8 @@ static int VCDRead( input_thread_t * p_input,
     }*/
 
     
-    b_eof = p_vcd->b_end_of_track && 
-            ( ( p_vcd->current_track + 1 ) >= p_vcd->nb_tracks );
+    b_eof = p_vcd->b_end_of_track; 
+        /*FIXME&& ( ( p_vcd->current_track ) >= p_vcd->nb_tracks - 1);*/
 
     if( b_eof )
     {
@@ -641,9 +644,9 @@ static int VCDRead( input_thread_t * p_input,
     if( p_vcd->b_end_of_track )
     {
         intf_WarnMsg( 4, "vcd info: new title" );
-        p_vcd->current_track++;
         p_vcd->b_end_of_track = 0;
-        VCDSetArea( p_input, p_input->stream.pp_areas[p_vcd->current_track] );
+        VCDSetArea( p_input, p_input->stream.pp_areas[
+                     p_input->stream.p_selected_area->i_id + 1] );
         vlc_mutex_unlock( &p_input->stream.stream_lock );
         return 0;
     }
