@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vout_xvideo.c: Xvideo video output display method
  *****************************************************************************
- * Copyright (C) 1998, 1999, 2000, 2001 VideoLAN
- * $Id: vout_xvideo.c,v 1.18 2001/05/30 17:03:12 sam Exp $
+ * Copyright (C) 1998-2001 VideoLAN
+ * $Id: vout_xvideo.c,v 1.36.2.2 2001/12/17 03:48:12 sam Exp $
  *
  * Authors: Shane Harper <shanegh@optusnet.com.au>
  *          Vincent Seguin <seguin@via.ecp.fr>
@@ -54,9 +54,10 @@
 #include <X11/extensions/XShm.h>
 #include <X11/extensions/Xv.h>
 #include <X11/extensions/Xvlib.h>
+#include <X11/extensions/dpms.h>
 
-#include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
@@ -65,12 +66,9 @@
 #include "video_output.h"
 
 #include "interface.h"
-#include "intf_msg.h"
 
 #include "netutils.h"                                 /* network_ChannelJoin */
 
-#include "main.h"
-
 #include "stream_control.h"                 /* needed by input_ext-intf.h... */
 #include "input_ext-intf.h"
 
@@ -132,6 +130,8 @@ typedef struct vout_sys_s
     /* Mouse pointer properties */
     boolean_t           b_mouse_pointer_visible;
     mtime_t             i_time_mouse_last_moved; /* used to auto-hide pointer*/
+    Cursor              blank_cursor;                   /* the hidden cursor */
+    Pixmap              cursor_pixmap;
 
 } vout_sys_t;
 
@@ -168,7 +168,7 @@ static int  XVideoCreateShmImage     ( Display* dpy, int xv_port,
                                        int i_width, int i_height );
 static void XVideoDestroyShmImage    ( vout_thread_t *, XvImage *,
                                        XShmSegmentInfo * );
-static void XVideoSetMousePointer    ( const vout_thread_t * );
+static void X11ToggleMousePointer    ( vout_thread_t * );
 static void XVideoEnableScreenSaver  ( vout_thread_t * );
 static void XVideoDisableScreenSaver ( vout_thread_t * );
 /*static void XVideoSetAttribute       ( vout_thread_t *, char *, float );*/
@@ -204,12 +204,42 @@ void _M( vout_getfunctions )( function_list_t * p_function_list )
  *****************************************************************************/
 static int vout_Probe( probedata_t *p_data )
 {
+    Display *p_display;                                   /* display pointer */
+    char    *psz_display;
+
+    /* Open display, unsing 'vlc_display' or DISPLAY environment variable */
+    psz_display = XDisplayName( main_GetPszVariable(VOUT_DISPLAY_VAR, NULL) );
+    p_display = XOpenDisplay( psz_display );
+    if( p_display == NULL )                                         /* error */
+    {
+        intf_WarnMsg( 3, "vout: Xvideo cannot open display %s", psz_display );
+        intf_WarnMsg( 3, "vout: Xvideo not supported" );
+        return( 0 );
+    }
+    
+    if( !XVideoCheckForXv( p_display ) )
+    {
+        intf_WarnMsg( 3, "vout: Xvideo not supported" );
+        XCloseDisplay( p_display );
+        return( 0 );
+    }
+
+    if( XVideoGetPort( p_display ) < 0 )
+    {
+        intf_WarnMsg( 3, "vout: Xvideo not supported" );
+        XCloseDisplay( p_display );
+        return( 0 );
+    }
+
+    /* Clean-up everyting */
+    XCloseDisplay( p_display );
+
     if( TestMethod( VOUT_METHOD_VAR, "xvideo" ) )
     {
         return( 999 );
     }
 
-    return( 60 );
+    return( 150 );
 }
 
 /*****************************************************************************
@@ -222,6 +252,7 @@ static int vout_Probe( probedata_t *p_data )
 static int vout_Create( vout_thread_t *p_vout )
 {
     char *psz_display;
+    XColor cursor_color;
 
     /* Allocate structure */
     p_vout->p_sys = malloc( sizeof( vout_sys_t ) );
@@ -254,6 +285,40 @@ static int vout_Create( vout_thread_t *p_vout )
         return( 1 );
     }
 
+    /* Check we have access to a video port */
+    if( (p_vout->p_sys->xv_port = XVideoGetPort(p_vout->p_sys->p_display)) <0 )
+    {
+        intf_ErrMsg( "vout error: cannot get XVideo port" );
+        XCloseDisplay( p_vout->p_sys->p_display );
+        free( p_vout->p_sys );
+        return 1;
+    }
+    intf_DbgMsg( "Using xv port %d" , p_vout->p_sys->xv_port );
+
+    /* Create blank cursor (for mouse cursor autohiding) */
+    p_vout->p_sys->b_mouse_pointer_visible = 1;
+    p_vout->p_sys->cursor_pixmap = XCreatePixmap( p_vout->p_sys->p_display,
+                                                  DefaultRootWindow(
+                                                     p_vout->p_sys->p_display),
+                                                  1, 1, 1 );
+    
+    XParseColor( p_vout->p_sys->p_display,
+                 XCreateColormap( p_vout->p_sys->p_display,
+                                  DefaultRootWindow(
+                                                    p_vout->p_sys->p_display ),
+                                  DefaultVisual(
+                                                p_vout->p_sys->p_display,
+                                                p_vout->p_sys->i_screen ),
+                                  AllocNone ),
+                 "black", &cursor_color );
+    
+    p_vout->p_sys->blank_cursor = XCreatePixmapCursor(
+                                      p_vout->p_sys->p_display,
+                                      p_vout->p_sys->cursor_pixmap,
+                                      p_vout->p_sys->cursor_pixmap,
+                                      &cursor_color,
+                                      &cursor_color, 1, 1 );    
+
     /* Spawn base window - this window will include the video output window,
      * but also command buttons, subtitles and other indicators */
     if( XVideoCreateWindow( p_vout ) )
@@ -264,9 +329,7 @@ static int vout_Create( vout_thread_t *p_vout )
         return( 1 );
     }
 
-    if( (p_vout->p_sys->xv_port = XVideoGetPort( p_vout->p_sys->p_display ))<0 )
-        return 1;
-    intf_DbgMsg( "Using xv port %d" , p_vout->p_sys->xv_port );
+    /* p_vout->pf_setbuffers( p_vout, NULL, NULL ); */
 
 #if 0
     /* XXX The brightness and contrast values should be read from environment
@@ -275,8 +338,6 @@ static int vout_Create( vout_thread_t *p_vout )
     XVideoSetAttribute( p_vout, "XV_CONTRAST",   0.5 );
 #endif
 
-    p_vout->p_sys->b_mouse_pointer_visible = 1;
-
     /* Disable screen saver and return */
     XVideoDisableScreenSaver( p_vout );
 
@@ -288,7 +349,7 @@ static int vout_Create( vout_thread_t *p_vout )
  *****************************************************************************/
 static int vout_Init( vout_thread_t *p_vout )
 {
-#ifdef SYS_DARWIN1_3
+#ifdef SYS_DARWIN
     /* FIXME : As of 2001-03-16, XFree4 for MacOS X does not support Xshm. */
     p_vout->p_sys->b_shm = 0;
 #endif
@@ -317,6 +378,13 @@ static void vout_End( vout_thread_t *p_vout )
  *****************************************************************************/
 static void vout_Destroy( vout_thread_t *p_vout )
 {
+    /* Restore cursor if it was blanked */
+    if( !p_vout->p_sys->b_mouse_pointer_visible )
+        X11ToggleMousePointer( p_vout );
+
+    /* Destroy blank cursor pixmap */
+    XFreePixmap( p_vout->p_sys->p_display, p_vout->p_sys->cursor_pixmap );
+
     XVideoEnableScreenSaver( p_vout );
     XVideoDestroyWindow( p_vout );
     XCloseDisplay( p_vout->p_sys->p_display );
@@ -335,6 +403,24 @@ static void vout_Destroy( vout_thread_t *p_vout )
  * XXX  Should "factor-out" common code in this and the "same" fn in the x11
  * XXX  plugin!
  *****************************************************************************/
+static __inline__ void vout_Seek( int i_seek )
+{
+    int i_tell = p_main->p_intf->p_input->stream.p_selected_area->i_tell;
+
+    i_tell += i_seek * 50 * p_main->p_intf->p_input->stream.i_mux_rate;
+
+    if( i_tell < p_main->p_intf->p_input->stream.p_selected_area->i_start )
+    {
+        i_tell = p_main->p_intf->p_input->stream.p_selected_area->i_start;
+    }
+    else if( i_tell > p_main->p_intf->p_input->stream.p_selected_area->i_size )
+    {
+        i_tell = p_main->p_intf->p_input->stream.p_selected_area->i_size;
+    }
+
+    input_Seek( p_main->p_intf->p_input, i_tell );
+}
+
 static int vout_Manage( vout_thread_t *p_vout )
 {
     XEvent      xevent;                                         /* X11 event */
@@ -359,6 +445,7 @@ static int vout_Manage( vout_thread_t *p_vout )
             /* Update dimensions */
             p_vout->p_sys->i_window_width = xevent.xconfigure.width;
             p_vout->p_sys->i_window_height = xevent.xconfigure.height;
+//            p_vout->i_changes |= VOUT_SIZE_CHANGE;
         }
         /* MapNotify event: change window status and disable screen saver */
         else if( xevent.type == MapNotify)
@@ -392,6 +479,36 @@ static int vout_Manage( vout_thread_t *p_vout )
                  case XK_Menu:
                      p_main->p_intf->b_menu_change = 1;
                      break;
+                 case XK_Left:
+                     vout_Seek( -5 );
+                     break;
+                 case XK_Right:
+                     vout_Seek( 5 );
+                     break;
+                 case XK_Up:
+                     vout_Seek( 60 );
+                     break;
+                 case XK_Down:
+                     vout_Seek( -60 );
+                     break;
+                 case XK_Home:
+                     input_Seek( p_main->p_intf->p_input,
+                     p_main->p_intf->p_input->stream.p_selected_area->i_start );                     break;
+                 case XK_End:
+                     input_Seek( p_main->p_intf->p_input,
+                     p_main->p_intf->p_input->stream.p_selected_area->i_size );
+                     break;
+                 case XK_Page_Up:
+                     vout_Seek( 900 );
+                     break;
+                 case XK_Page_Down:
+                     vout_Seek( -900 );
+                     break;
+                 case XK_space:
+                     input_SetStatus( p_main->p_intf->p_input,
+                                      INPUT_STATUS_PAUSE );
+                     break;
+
                  default:
                      /* "Normal Keys"
                       * The reason why I use this instead of XK_0 is that 
@@ -459,8 +576,11 @@ static int vout_Manage( vout_thread_t *p_vout )
             switch( ((XButtonEvent *)&xevent)->button )
             {
                 case Button1:
-                    /* in this part we will eventually manage
-                     * clicks for DVD navigation for instance */
+                    /* In this part we will eventually manage
+                     * clicks for DVD navigation for instance. For the
+                     * moment just pause the stream. */
+                    input_SetStatus( p_main->p_intf->p_input,
+                                     INPUT_STATUS_PAUSE );
                     break;
             }
         }
@@ -479,8 +599,8 @@ static int vout_Manage( vout_thread_t *p_vout )
         else if( xevent.type == MotionNotify )
         {
             p_vout->p_sys->i_time_mouse_last_moved = mdate();
-            p_vout->p_sys->b_mouse_pointer_visible = 1; 
-            XVideoSetMousePointer( p_vout ); 
+            if( !p_vout->p_sys->b_mouse_pointer_visible )
+                X11ToggleMousePointer( p_vout ); 
         }
         /* Other event */
         else
@@ -557,19 +677,19 @@ static int vout_Manage( vout_thread_t *p_vout )
     {
         intf_DbgMsg( "vout: resizing window" );
         p_vout->i_changes &= ~VOUT_SIZE_CHANGE;
-        /* Nothing to do here...
-         * vout_Display() detects size changes of the image to be displayed and
-         * re-creates the XvImage.*/
-        intf_Msg( "vout: video display resized (%dx%d)",
-                  p_vout->i_width, p_vout->i_height );
+
+        p_vout->i_width = p_vout->p_sys->i_window_width;
+        p_vout->i_height = p_vout->p_sys->i_window_height;
+
+        intf_WarnMsg( 3, "vout: video display resized (%dx%d)",
+                      p_vout->i_width, p_vout->i_height );
     }
 
     /* Autohide Cursor */
     if( p_vout->p_sys->b_mouse_pointer_visible &&
         mdate() - p_vout->p_sys->i_time_mouse_last_moved > 2000000 )
     {
-        p_vout->p_sys->b_mouse_pointer_visible = 0; 
-        XVideoSetMousePointer( p_vout );
+        X11ToggleMousePointer( p_vout );
     }
     
     return 0;
@@ -588,7 +708,9 @@ static void vout_Display( vout_thread_t *p_vout )
                    p_vout->p_rendered_pic->i_height;
 
     if( XVideoUpdateImgSizeIfRequired( p_vout ) )
+    {
         return;
+    }
 
     switch( p_vout->p_rendered_pic->i_type )
     {
@@ -603,17 +725,20 @@ static void vout_Display( vout_thread_t *p_vout )
         break;
 
     case YUV_420_PICTURE:
-        memcpy( p_vout->p_sys->p_xvimage->data,
-                p_vout->p_rendered_pic->p_y, i_size );
-        memcpy( p_vout->p_sys->p_xvimage->data + ( i_size ),
-                p_vout->p_rendered_pic->p_v, i_size / 4 );
-        memcpy( p_vout->p_sys->p_xvimage->data + ( i_size ) + ( i_size / 4 ),
-                p_vout->p_rendered_pic->p_u, i_size / 4 );
+        p_main->fast_memcpy( p_vout->p_sys->p_xvimage->data,
+                             p_vout->p_rendered_pic->p_y, i_size );
+        p_main->fast_memcpy( p_vout->p_sys->p_xvimage->data + ( i_size ),
+                             p_vout->p_rendered_pic->p_v, i_size / 4 );
+        p_main->fast_memcpy( p_vout->p_sys->p_xvimage->data
+                                 + ( i_size ) + ( i_size / 4 ),
+                             p_vout->p_rendered_pic->p_u, i_size / 4 );
         break;
     }
 
     if( b_draw )
+    {
         XVideoDisplay( p_vout );
+    }
 }
 
 static void vout_SetPalette( p_vout_thread_t p_vout,
@@ -641,13 +766,16 @@ static int XVideoUpdateImgSizeIfRequired( vout_thread_t *p_vout )
     if( p_vout->p_sys->i_image_width != i_img_width
             || p_vout->p_sys->i_image_height != i_img_height )
     {
+        if( p_vout->p_sys->i_image_width != 0
+             && p_vout->p_sys->i_image_height != 0 )
+        {
+            /* Destroy XvImage to change its size */
+            vout_End( p_vout );
+        }
+
         p_vout->p_sys->i_image_width  = i_img_width;
         p_vout->p_sys->i_image_height = i_img_height;
 
-        /* Destroy XvImage to change its size */
-        vout_End( p_vout );
-            /* Note: vout_End does nothing if no XvImage to destroy. */
-
         /* Create XvImage using XShm extension */
         if( XVideoCreateShmImage( p_vout->p_sys->p_display,
                                   p_vout->p_sys->xv_port,
@@ -665,7 +793,6 @@ static int XVideoUpdateImgSizeIfRequired( vout_thread_t *p_vout )
             (p_vout->p_sys->p_xvimage->data_size) /
             (p_vout->p_sys->p_xvimage->height);
 
-        /* p_vout->pf_setbuffers( p_vout, p_vout->p_sys->p_xvimage->data ); */
     }
 
     return( 0 );
@@ -684,15 +811,15 @@ static int XVideoCheckForXv( Display *dpy )
             return( 1 );
 
         case XvBadExtension:
-            intf_ErrMsg( "vout error: XvBadExtension" );
+            intf_WarnMsg( 3, "vout error: XvBadExtension" );
             return( 0 );
 
         case XvBadAlloc:
-            intf_ErrMsg( "vout error: XvBadAlloc" );
+            intf_WarnMsg( 3, "vout error: XvBadAlloc" );
             return( 0 );
 
         default:
-            intf_ErrMsg( "vout error: XvQueryExtension failed" );
+            intf_WarnMsg( 3, "vout error: XvQueryExtension failed" );
             return( 0 );
     }
 }
@@ -722,15 +849,11 @@ static int XVideoCreateWindow( vout_thread_t *p_vout )
                                                       p_vout->p_sys->i_screen );
         p_vout->p_sys->i_window_height =  DisplayHeight( p_vout->p_sys->p_display,
                                                          p_vout->p_sys->i_screen );
-/*      p_vout->i_width =  p_vout->p_sys->i_window_width;
-        p_vout->i_height = p_vout->p_sys->i_window_height; */
     }
     else
     {
-        p_vout->p_sys->i_window_width =  main_GetIntVariable( VOUT_WIDTH_VAR,
-                                                       VOUT_WIDTH_DEFAULT );
-        p_vout->p_sys->i_window_height = main_GetIntVariable( VOUT_HEIGHT_VAR,
-                                                       VOUT_HEIGHT_DEFAULT );
+        p_vout->p_sys->i_window_width =  p_vout->i_width;
+        p_vout->p_sys->i_window_height = p_vout->i_height;
     }
 
     /* Prepare window manager hints and properties */
@@ -862,14 +985,22 @@ static int XVideoCreateWindow( vout_thread_t *p_vout )
     XSelectInput( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
                   ExposureMask );
 
+    /* If the cursor was formerly blank than blank it again */
+    if( !p_vout->p_sys->b_mouse_pointer_visible )
+    {
+        X11ToggleMousePointer( p_vout );
+        X11ToggleMousePointer( p_vout );
+    }
 
-    XVideoSetMousePointer( p_vout );
+    XSync( p_vout->p_sys->p_display, False );
 
     return( 0 );
 }
 
 static void XVideoDestroyWindow( vout_thread_t *p_vout )
 {
+    XSync( p_vout->p_sys->p_display, False );
+
     XFreeGC( p_vout->p_sys->p_display, p_vout->p_sys->yuv_gc );
     XDestroyWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window );
 
@@ -914,20 +1045,21 @@ static int XVideoCreateShmImage( Display* dpy, int xv_port,
                                                         0, 0 );
     p_shm_info->readOnly = False;
 
-    /* Mark the shm segment to be removed when there will be no more
-     * attachements, so it is automatic on process exit or after shmdt */
-    shmctl( p_shm_info->shmid, IPC_RMID, 0 );
-
     if( !XShmAttach( dpy, p_shm_info ) )
     {
         intf_ErrMsg( "vout error: XShmAttach failed" );
+        shmctl( p_shm_info->shmid, IPC_RMID, 0 );
         shmdt( p_shm_info->shmaddr );
         return( -1 );
     }
 
     /* Send image to X server. This instruction is required, since having
      * built a Shm XImage and not using it causes an error on XCloseDisplay */
-    XFlush( dpy );
+    XSync( dpy, False );
+
+    /* Mark the shm segment to be removed when there will be no more
+     * attachements, so it is automatic on process exit or after shmdt */
+    shmctl( p_shm_info->shmid, IPC_RMID, 0 );
 
     return( 0 );
 }
@@ -948,13 +1080,15 @@ static void XVideoDestroyShmImage( vout_thread_t *p_vout, XvImage *p_xvimage,
         return;
     }
 
+    XSync( p_vout->p_sys->p_display, False );
     XShmDetach( p_vout->p_sys->p_display, p_shm_info );/* detach from server */
 #if 0
     XDestroyImage( p_ximage ); /* XXX */
 #endif
+    XFree( p_xvimage );
 
     if( shmdt( p_shm_info->shmaddr ) )  /* detach shared memory from process */
-    {                                   /* also automatic freeing...         */
+    {
         intf_ErrMsg( "vout error: cannot detach shared memory (%s)",
                      strerror(errno) );
     }
@@ -976,6 +1110,8 @@ void XVideoEnableScreenSaver( vout_thread_t *p_vout )
                      p_vout->p_sys->i_ss_interval,
                      p_vout->p_sys->i_ss_blanking,
                      p_vout->p_sys->i_ss_exposure );
+
+    DPMSEnable( p_vout->p_sys->p_display );
 }
 
 /*****************************************************************************
@@ -997,48 +1133,29 @@ void XVideoDisableScreenSaver( vout_thread_t *p_vout )
                      p_vout->p_sys->i_ss_interval,
                      p_vout->p_sys->i_ss_blanking,
                      p_vout->p_sys->i_ss_exposure );
+
+    DPMSDisable( p_vout->p_sys->p_display );
 }
 
 /*****************************************************************************
- * XVideoSetMousePointer: hide or show the mouse pointer
+ * X11ToggleMousePointer: hide or show the mouse pointer
  *****************************************************************************
  * This function hides the X pointer if requested.
  *****************************************************************************/
-void XVideoSetMousePointer( const vout_thread_t *p_vout )
+void X11ToggleMousePointer( vout_thread_t *p_vout )
 {
-    static Cursor blank_cursor;
-    static boolean_t b_created_blank_cursor = 0;
 
-    if( !p_vout->p_sys->b_mouse_pointer_visible )
+    if( p_vout->p_sys->b_mouse_pointer_visible )
     {
-        if( !b_created_blank_cursor )
-        {
-            XColor color;
-            Pixmap blank = XCreatePixmap( p_vout->p_sys->p_display,
-                               DefaultRootWindow(p_vout->p_sys->p_display),
-                               1, 1, 1 );
-
-            XParseColor( p_vout->p_sys->p_display,
-                         XCreateColormap( p_vout->p_sys->p_display,
-                                          DefaultRootWindow(
-                                                  p_vout->p_sys->p_display ),
-                                          DefaultVisual(
-                                                  p_vout->p_sys->p_display,
-                                                  p_vout->p_sys->i_screen ),
-                                          AllocNone ),
-                         "black", &color );
-
-            blank_cursor = XCreatePixmapCursor( p_vout->p_sys->p_display,
-                           blank, blank, &color, &color, 1, 1 );
-
-            b_created_blank_cursor = 1;
-        }
         XDefineCursor( p_vout->p_sys->p_display,
-                       p_vout->p_sys->window, blank_cursor );
+                       p_vout->p_sys->window,
+                       p_vout->p_sys->blank_cursor );
+        p_vout->p_sys->b_mouse_pointer_visible = 0;
     }
     else
     {
         XUndefineCursor( p_vout->p_sys->p_display, p_vout->p_sys->window );
+        p_vout->p_sys->b_mouse_pointer_visible = 1;
     }
 }
 
@@ -1046,123 +1163,173 @@ void XVideoSetMousePointer( const vout_thread_t *p_vout )
  * used by the xvideo plugin, but others may want to use it. */
 static void XVideoOutputCoords( const picture_t *p_pic, const boolean_t scale,
                                 const int win_w, const int win_h,
-                                int *dx, int *dy, int *w, int *h)
+                                int *dx, int *dy, int *w, int *h )
 {
-    if( !scale )
-    {
-        *w = p_pic->i_width; *h = p_pic->i_height;
-    }
-    else
-    {
-        *w = win_w;
-        switch( p_pic->i_aspect_ratio )
-        {
-            case AR_3_4_PICTURE:
-                *h = win_w * 3 / 4;
-                break;
-
-            case AR_16_9_PICTURE:
-                *h = win_w * 9 / 16;
-                break;
-
-            case AR_221_1_PICTURE:
-                *h = win_w * 100 / 221;
-                break;
-
-            case AR_SQUARE_PICTURE:
-            default:
-                *h = win_w * p_pic->i_height / p_pic->i_width;
-                break;
-        }
-
-        if( *h > win_h )
-        {
-            *h = win_h;
-            switch( p_pic->i_aspect_ratio )
-            {
-                case AR_3_4_PICTURE:
-                    *w = win_h * 4 / 3;
-                    break;
-
-                case AR_16_9_PICTURE:
-                    *w = win_h * 16 / 9;
-                    break;
-
-                case AR_221_1_PICTURE:
-                    *w = win_h * 221 / 100;
-                    break;
-
-                case AR_SQUARE_PICTURE:
-                default:
-                    *w = win_h * p_pic->i_width / p_pic->i_height;
-                    break;
-            }
-        }
-    }
-
-    /* Set picture position */
-    *dx = (win_w - *w) / 2;
-    *dy = (win_h - *h) / 2;
 }
 
 
+/*****************************************************************************
+ * XVideoGetPort: get YUV12 port
+ *****************************************************************************
+ * 
+ *****************************************************************************/
 static int XVideoGetPort( Display *dpy )
 {
-    int            i, i_adaptors;
-    int            xv_port = -1;
-    XvAdaptorInfo *adaptor_info;
+    XvAdaptorInfo *p_adaptor;
+    int i_adaptor, i_num_adaptors, i_requested_adaptor;
+    int i_selected_port;
 
     switch( XvQueryAdaptors( dpy, DefaultRootWindow( dpy ),
-                             &i_adaptors, &adaptor_info ) )
+                             &i_num_adaptors, &p_adaptor ) )
     {
         case Success:
             break;
 
         case XvBadExtension:
-            intf_ErrMsg( "vout error: XvBadExtension for XvQueryAdaptors" );
+            intf_WarnMsg( 3, "vout error: XvBadExtension for XvQueryAdaptors" );
             return( -1 );
 
         case XvBadAlloc:
-            intf_ErrMsg( "vout error: XvBadAlloc for XvQueryAdaptors" );
+            intf_WarnMsg( 3, "vout error: XvBadAlloc for XvQueryAdaptors" );
             return( -1 );
 
         default:
-            intf_ErrMsg( "vout error: XvQueryAdaptors failed" );
+            intf_WarnMsg( 3, "vout error: XvQueryAdaptors failed" );
             return( -1 );
     }
 
-    for( i=0; i < i_adaptors && xv_port == -1; ++i )
-        if( ( adaptor_info[ i ].type & XvInputMask ) &&
-            ( adaptor_info[ i ].type & XvImageMask ) )
+    i_selected_port = -1;
+    i_requested_adaptor = main_GetIntVariable( VOUT_XVADAPTOR_VAR, -1 );
+
+    /* No special xv port has been requested so try all of them */
+    for( i_adaptor = 0; i_adaptor < i_num_adaptors; ++i_adaptor )
+    {
+        XvImageFormatValues *p_formats;
+        int i_format, i_num_formats;
+        int i_port;
+
+        /* If we requested an adaptor and it's not this one, we aren't
+         * interested */
+        if( i_requested_adaptor != -1 && i_adaptor != i_requested_adaptor )
+	{
+            continue;
+	}
+
+        /* Check that port supports YUV12 planar format... */
+        p_formats = XvListImageFormats( dpy, p_adaptor[i_adaptor].base_id,
+                                        &i_num_formats );
+
+        for( i_format = 0; i_format < i_num_formats; i_format++ )
         {
-            /* check that port supports YUV12 planar format... */
-            int port = adaptor_info[ i ].base_id;
-            int i_num_formats, i;
-            XvImageFormatValues *imageFormats;
+            XvEncodingInfo  *p_enc;
+            int             i_enc, i_num_encodings;
+            XvAttribute     *p_attr;
+            int             i_attr, i_num_attributes;
 
-            imageFormats = XvListImageFormats( dpy, port, &i_num_formats );
+            /* If this is not the format we want, forget it */
+            if( p_formats[ i_format ].id != GUID_YUV12_PLANAR )
+            {
+                continue;
+            }
 
-            for( i=0; i < i_num_formats && xv_port == -1; ++i )
-                if( imageFormats[ i ].id == GUID_YUV12_PLANAR )
-                    xv_port = port;
+            /* Look for the first available port supporting this format */
+            for( i_port = p_adaptor[i_adaptor].base_id;
+                 ( i_port < p_adaptor[i_adaptor].base_id
+                             + p_adaptor[i_adaptor].num_ports )
+                   && ( i_selected_port == -1 );
+                 i_port++ )
+            {
+                if( XvGrabPort( dpy, i_port, CurrentTime ) == Success )
+                {
+                    i_selected_port = i_port;
+                }
+            }
 
-            if( xv_port == -1 )
-                intf_WarnMsg( 3, "vout: XVideo image input port %d "
-                        "does not support the YUV12 planar format which is "
-                        "currently required by the xvideo output plugin.",
-                        port );
+            /* If no free port was found, forget it */
+            if( i_selected_port == -1 )
+            {
+                continue;
+            }
 
-            if( imageFormats )
-                XFree( imageFormats );
+            /* If we found a port, print information about it */
+            intf_WarnMsg( 3, "vout: GetXVideoPort found adaptor %i, port %i",
+                             i_adaptor, i_selected_port );
+            intf_WarnMsg( 3, "  image format 0x%x (%4.4s) %s supported",
+                             p_formats[ i_format ].id,
+                             (char *)&p_formats[ i_format ].id,
+                             ( p_formats[ i_format ].format
+                                == XvPacked ) ? "packed" : "planar" );
+
+            intf_WarnMsg( 4, " encoding list:" );
+
+            if( XvQueryEncodings( dpy, i_selected_port,
+                                  &i_num_encodings, &p_enc )
+                 != Success )
+            {
+                intf_WarnMsg( 4, "  XvQueryEncodings failed" );
+                continue;
+            }
+
+            for( i_enc = 0; i_enc < i_num_encodings; i_enc++ )
+            {
+                intf_WarnMsg( 4, "  id=%ld, name=%s, size=%ldx%ld,"
+                                 " numerator=%d, denominator=%d",
+                              p_enc[i_enc].encoding_id, p_enc[i_enc].name,
+                              p_enc[i_enc].width, p_enc[i_enc].height,
+                              p_enc[i_enc].rate.numerator,
+                              p_enc[i_enc].rate.denominator );
+            }
+
+            if( p_enc != NULL )
+            {
+                XvFreeEncodingInfo( p_enc );
+            }
+
+            intf_WarnMsg( 4, " attribute list:" );
+            p_attr = XvQueryPortAttributes( dpy, i_selected_port,
+                                            &i_num_attributes );
+            for( i_attr = 0; i_attr < i_num_attributes; i_attr++ )
+            {
+                intf_WarnMsg( 4,
+                      "  name=%s, flags=[%s%s ], min=%i, max=%i",
+                      p_attr[i_attr].name,
+                      (p_attr[i_attr].flags & XvGettable) ? " get" : "",
+                      (p_attr[i_attr].flags & XvSettable) ? " set" : "",
+                      p_attr[i_attr].min_value, p_attr[i_attr].max_value );
+            }
+
+            if( p_attr != NULL )
+            {
+                XFree( p_attr );
+            }
         }
 
-    if( i_adaptors > 0 )
-        XvFreeAdaptorInfo(adaptor_info);
+        if( p_formats != NULL )
+        {
+            XFree( p_formats );
+        }
 
-    if( xv_port == -1 )
-        intf_ErrMsg( "vout error: didn't find a suitable Xvideo image input port." );
+    }
 
-    return( xv_port );
+    if( i_num_adaptors > 0 )
+    {
+        XvFreeAdaptorInfo( p_adaptor );
+    }
+
+    if( i_selected_port == -1 )
+    {
+        if( i_requested_adaptor == -1 )
+        {
+            intf_WarnMsg( 3, "vout: no free XVideo port found for YV12" );
+        }
+        else
+        {
+            intf_WarnMsg( 3, "vout: XVideo adaptor %i does not have a free "
+                             "XVideo port for YV12", i_requested_adaptor );
+        }
+    }
+
+    return( i_selected_port );
 }
 
 
@@ -1175,15 +1342,94 @@ static int XVideoGetPort( Display *dpy )
  *****************************************************************************/
 static void XVideoDisplay( vout_thread_t *p_vout )
 {
-    int     i_dest_width, i_dest_height, i_dest_x, i_dest_y;
+    int         i_dest_width, i_dest_height;
+    int         i_dest_x, i_dest_y;
 
-    if( !p_vout->p_sys->p_xvimage ) return;
+    if( !p_vout->p_sys->p_xvimage || !p_vout->p_rendered_pic )
+    {
+        return;
+    }
 
-    XVideoOutputCoords( p_vout->p_rendered_pic, p_vout->b_scale,
-                        p_vout->p_sys->i_window_width,
-                        p_vout->p_sys->i_window_height,
-                        &i_dest_x, &i_dest_y,
-                        &i_dest_width, &i_dest_height);
+    i_dest_height = p_vout->p_sys->i_window_height >
+                        p_vout->p_rendered_pic->i_height
+                  ? p_vout->p_sys->i_window_height
+                  : p_vout->p_rendered_pic->i_height;
+    i_dest_width = p_vout->p_sys->i_window_width >
+                        p_vout->p_rendered_pic->i_width
+                 ? p_vout->p_sys->i_window_width
+                 : p_vout->p_rendered_pic->i_width;
+        
+    if( p_vout->b_scale )
+    {
+        int   i_ratio = 900 * i_dest_width / i_dest_height;
+        
+        switch( p_vout->p_rendered_pic->i_aspect_ratio )
+        {
+            case AR_3_4_PICTURE:
+                if( i_ratio < 1200 )
+                {
+                    i_dest_width = i_dest_height * 4 / 3;
+                }
+                else
+                {
+                    i_dest_height = i_dest_width * 3 / 4;
+                }
+                i_ratio = 1200;
+                break;
+
+            case AR_16_9_PICTURE:
+                if( i_ratio < 1600 )
+                {
+                    i_dest_width = i_dest_height * 16 / 9;
+                }
+                else
+                {
+                    i_dest_height = i_dest_width * 9 / 16;
+                }
+                i_ratio = 1600;
+                break;
+
+            case AR_221_1_PICTURE:
+                if( i_ratio < 1989 )
+                {
+                    i_dest_width = i_dest_height * 221 / 100;
+                }
+                else
+                {
+                    i_dest_height = i_dest_width * 100 / 221;
+                }
+                i_ratio = 1989;
+                break;
+
+            case AR_SQUARE_PICTURE:
+            default:
+                if( i_ratio < 900 )
+                {
+                    i_dest_width = i_dest_height * p_vout->p_rendered_pic->i_width / p_vout->p_rendered_pic->i_height;
+                }
+                else
+                {
+                    i_dest_height = i_dest_width * p_vout->p_rendered_pic->i_height / p_vout->p_rendered_pic->i_width;
+                }
+                i_ratio = 900;
+                break;
+        }
+
+        if( i_dest_width >
+            DisplayWidth( p_vout->p_sys->p_display, p_vout->p_sys->i_screen ) )
+        {
+            i_dest_width = DisplayWidth( p_vout->p_sys->p_display,
+                                         p_vout->p_sys->i_screen );
+            i_dest_height = 900 * i_dest_width / i_ratio;
+        }
+        else if( i_dest_height >
+            DisplayHeight( p_vout->p_sys->p_display, p_vout->p_sys->i_screen ) )
+        {
+            i_dest_height = DisplayHeight( p_vout->p_sys->p_display,
+                                           p_vout->p_sys->i_screen );
+            i_dest_width = i_ratio * i_dest_height / 900;
+        }
+    }
 
     XvShmPutImage( p_vout->p_sys->p_display, p_vout->p_sys->xv_port,
                    p_vout->p_sys->yuv_window, p_vout->p_sys->gc,
@@ -1192,12 +1438,33 @@ static void XVideoDisplay( vout_thread_t *p_vout )
                    p_vout->p_rendered_pic->i_width,
                    p_vout->p_rendered_pic->i_height,
                    0 /*dest_x*/, 0 /*dest_y*/, i_dest_width, i_dest_height,
-                   True );
+                   False );
+    
+    /* YUV window */
+    XResizeWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
+                   i_dest_width, i_dest_height );
 
-     XResizeWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
-             i_dest_width, i_dest_height );
-     XMoveWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
-             i_dest_x, i_dest_y );
+    /* Root window */
+    if( ( ( i_dest_width != p_vout->p_sys->i_window_width ) ||
+          ( i_dest_height != p_vout->p_sys->i_window_height ) ) &&
+        ! p_vout->b_fullscreen )
+    {
+        p_vout->p_sys->i_window_width = i_dest_width;
+        p_vout->p_sys->i_window_height = i_dest_height;
+//        p_vout->i_changes |= VOUT_SIZE_CHANGE;
+        XResizeWindow( p_vout->p_sys->p_display, p_vout->p_sys->window,
+                       i_dest_width, i_dest_height );
+    }
+    
+    /* Set picture position */
+    i_dest_x = (p_vout->p_sys->i_window_width - i_dest_width) / 2;
+    i_dest_y = (p_vout->p_sys->i_window_height - i_dest_height) / 2;
+    
+    XMoveWindow( p_vout->p_sys->p_display, p_vout->p_sys->yuv_window,
+                 i_dest_x, i_dest_y );
+    
+    /* Send the order to the X server */
+    XSync( p_vout->p_sys->p_display, False );
 }
 
 #if 0
@@ -1238,3 +1505,4 @@ static void XVideoSetAttribute( vout_thread_t *p_vout,
         XFree( p_attrib );
 }
 #endif
+
