@@ -2,8 +2,8 @@
  * transforms_yuvmmx.c: MMX YUV transformation functions
  * Provides functions to perform the YUV conversion.
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
- * $Id: transforms_yuvmmx.c,v 1.5 2001/04/15 04:19:58 sam Exp $
+ * Copyright (C) 1999-2001 VideoLAN
+ * $Id: transforms_yuvmmx.c,v 1.7 2001/11/28 15:08:06 massiot Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -38,6 +38,7 @@
 
 #include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 
@@ -48,7 +49,8 @@
 #include "transforms_common.h"
 #include "transforms_yuvmmx.h"
 
-#include "intf_msg.h"
+#include "modules.h"
+#include "modules_export.h"
 
 /*****************************************************************************
  * ConvertY4Gray8: grayscale YUV 4:x:x to RGB 8 bpp
