@@ -1,8 +1,8 @@
 /*****************************************************************************
  * mpeg_system.c: TS, PS and PES management
  *****************************************************************************
- * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: mpeg_system.c,v 1.64 2001/11/12 10:33:42 massiot Exp $
+ * Copyright (C) 1998-2001 VideoLAN
+ * $Id: mpeg_system.c,v 1.69.2.4 2001/12/17 16:37:25 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Michel Lespinasse <walken@via.ecp.fr>
@@ -34,20 +34,16 @@
 #include <string.h>                                    /* memcpy(), memset() */
 #include <sys/types.h>                                              /* off_t */
 
-#include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 
-#include "intf_msg.h"
-
 #include "stream_control.h"
 #include "input_ext-intf.h"
 #include "input_ext-dec.h"
 #include "input_ext-plugins.h"
 
-#include "main.h"                   /* AC3/MPEG channel, SPU channel, b_stat */
-
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
@@ -633,9 +629,7 @@ static void DecodePSM( input_thread_t * p_input, data_packet_t * p_data )
     }
 
     /* Un-select the streams that are no longer parts of the program. */
-    for( i = i_new_es_number;
-         i < p_input->stream.pp_programs[0]->i_es_number;
-         i++ )
+    while( i_new_es_number < p_input->stream.pp_programs[0]->i_es_number )
     {
         /* We remove pp_es[i_new_es_member] and not pp_es[i] because the
          * list will be emptied starting from the end */
@@ -780,6 +774,9 @@ es_descriptor_t * input_ParsePS( input_thread_t * p_input,
                         p_es->i_type = UNKNOWN_ES;
                     }
                 }
+
+                /* Tell the interface the stream has changed */
+                p_input->stream.b_changed = 1;
             }
         } /* stream.b_is_ok */
         vlc_mutex_unlock( &p_input->stream.stream_lock );
@@ -1278,7 +1275,7 @@ void input_DemuxPSI( input_thread_t * p_input, data_packet_t * p_data,
         else
         {
             memcpy( p_psi->buffer, p, p_data->p_payload_end - p );
-            p_psi->i_read_in_section+= p_data->p_payload_end - p;
+            p_psi->i_read_in_section += p_data->p_payload_end - p;
 
             p_psi->p_current += p_data->p_payload_end - p;
         }
@@ -1297,51 +1294,106 @@ void input_DemuxPSI( input_thread_t * p_input, data_packet_t * p_data,
  *****************************************************************************/
 static void input_DecodePAT( input_thread_t * p_input, es_descriptor_t * p_es )
 {
-    
     stream_ts_data_t  * p_stream_data;
     es_ts_data_t      * p_demux_data;
 
+    pgrm_descriptor_t * p_pgrm;
+    es_descriptor_t   * p_current_es;
+    byte_t            * p_current_data;           
+
+    int                 i_section_length, i_program_id, i_pmt_pid;
+    int                 i_loop, i_current_section;
+
+    boolean_t           b_changed = 0;
+
     p_demux_data = (es_ts_data_t *)p_es->p_demux_data;
     p_stream_data = (stream_ts_data_t *)p_input->stream.p_demux_data;
     
 #define p_psi (p_demux_data->p_psi_section)
 
+    /* Not so fast, Mike ! If the PAT version has changed, we first check
+     * that its content has really changed before doing anything */
     if( p_stream_data->i_pat_version != p_psi->i_version_number )
     {
-        /* PAT has changed. We are going to delete all programms and 
+        int i_programs = p_input->stream.i_pgrm_number;
+
+        p_current_data = p_psi->buffer;
+
+        do
+        {
+            i_section_length = ((u32)(p_current_data[1] & 0xF) << 8) |
+                                 p_current_data[2];
+            i_current_section = (u8)p_current_data[6];
+    
+            for( i_loop = 0;
+                 ( i_loop < (i_section_length - 9) / 4 ) && !b_changed;
+                 i_loop++ )
+            {
+                i_program_id = ( (u32)*(p_current_data + i_loop * 4 + 8) << 8 )
+                                 | *(p_current_data + i_loop * 4 + 9);
+                i_pmt_pid = ( ((u32)*(p_current_data + i_loop * 4 + 10) & 0x1F)
+                                    << 8 )
+                               | *(p_current_data + i_loop * 4 + 11);
+
+                if( i_program_id )
+                {
+                    if( (p_pgrm = input_FindProgram( p_input, i_program_id ))
+                        && (p_current_es = input_FindES( p_input, i_pmt_pid ))
+                        && p_current_es->p_pgrm == p_pgrm
+                        && p_current_es->i_id == i_pmt_pid
+                        && ((es_ts_data_t *)p_current_es->p_demux_data)->b_psi
+                        && ((es_ts_data_t *)p_current_es->p_demux_data)
+                            ->i_psi_type == PSI_IS_PMT )
+                    {
+                        i_programs--;
+                    }
+                    else
+                    {
+                        b_changed = 1;
+                    }
+                }
+            }
+            
+            p_current_data += 3 + i_section_length;
+
+        } while( ( i_current_section < p_psi->i_last_section_number )
+                  && !b_changed );
+
+        /* If we didn't find the expected amount of programs, the PAT has
+         * changed. Otherwise, it only changed if b_changed is already != 0 */
+        b_changed = b_changed || i_programs;
+    }
+
+    if( b_changed )
+    {
+        /* PAT has changed. We are going to delete all programs and 
          * create new ones. We chose not to only change what was needed
          * as a PAT change may mean the stream is radically changing and
-         * this is a secure method to avoid krashes */
-        pgrm_descriptor_t * p_pgrm;
-        es_descriptor_t   * p_current_es;
+         * this is a secure method to avoid crashes */
         es_ts_data_t      * p_es_demux;
         pgrm_ts_data_t    * p_pgrm_demux;
-        byte_t            * p_current_data;           
-        
-        int                 i_section_length,i_program_id,i_pmt_pid;
-        int                 i_loop, i_current_section;
         
         p_current_data = p_psi->buffer;
 
-
-        for( i_loop = 0; i_loop < p_input->stream.i_pgrm_number; i_loop++ )
+        /* Delete all programs */
+        while( p_input->stream.i_pgrm_number )
         {
-            input_DelProgram( p_input, p_input->stream.pp_programs[i_loop] );
+            input_DelProgram( p_input, p_input->stream.pp_programs[0] );
         }
         
         do
         {
-            i_section_length = ((p_current_data[1] & 0xF) << 8) |
+            i_section_length = ((u32)(p_current_data[1] & 0xF) << 8) |
                                  p_current_data[2];
             i_current_section = (u8)p_current_data[6];
     
-            for( i_loop = 0; i_loop < (i_section_length-9)/4 ; i_loop++ )
+            for( i_loop = 0; i_loop < (i_section_length - 9) / 4 ; i_loop++ )
             {
-                i_program_id = ( *(p_current_data + i_loop * 4 + 8) << 8 ) |
-                                 *(p_current_data + i_loop * 4 + 9);
-                i_pmt_pid = ( (*( p_current_data + i_loop * 4 + 10) & 0x1F)
-                                    << 8 ) |
-                               *( p_current_data + i_loop * 4 + 11);
+                i_program_id = ( (u32)*(p_current_data + i_loop * 4 + 8) << 8 )
+                                 | *(p_current_data + i_loop * 4 + 9);
+                i_pmt_pid = ( ((u32)*(p_current_data + i_loop * 4 + 10) & 0x1F)
+                                    << 8 )
+                               | *(p_current_data + i_loop * 4 + 11);
     
                 /* If program = 0, we're having info about NIT not PMT */
                 if( i_program_id )
@@ -1367,16 +1419,18 @@ static void input_DecodePAT( input_thread_t * p_input, es_descriptor_t * p_es )
                 }
             }
             
-            p_current_data+=3+i_section_length;
-            
+            p_current_data += 3 + i_section_length;
+
         } while( i_current_section < p_psi->i_last_section_number );
-        
-        /* Go to the beginning of the next section*/
+
+        /* Go to the beginning of the next section */
         p_stream_data->i_pat_version = p_psi->i_version_number;
 
     }
 #undef p_psi    
 
+    /* FIXME This has nothing to do here */
+    p_input->stream.p_selected_program = p_input->stream.pp_programs[0] ;
 }
 
 /*****************************************************************************
@@ -1411,7 +1465,7 @@ static void input_DecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
         p_current_section = p_psi->buffer;
         p_current_data = p_psi->buffer;
 
-        p_pgrm_data->i_pcr_pid = ( (*(p_current_section + 8) & 0x1F) << 8 ) |
+        p_pgrm_data->i_pcr_pid = ( ((u32)*(p_current_section + 8) & 0x1F) << 8 ) |
                                     *(p_current_section + 9);
 
         i_audio_es = 0;
@@ -1452,22 +1506,26 @@ static void input_DecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
             i_required_spu_es = 0;
         }
         
-        /* Delete all ES in this program  except the PSI */
-        for( i_loop=0; i_loop < p_es->p_pgrm->i_es_number; i_loop++ )
+        /* Delete all ES in this program  except the PSI. We start from the
+         * end because i_es_number gets decremented after each deletion. */
+        for( i_loop = p_es->p_pgrm->i_es_number ; i_loop ; )
         {
+            i_loop--;
             p_es_demux = (es_ts_data_t *)
                          p_es->p_pgrm->pp_es[i_loop]->p_demux_data;
             if ( ! p_es_demux->b_psi )
-            input_DelES( p_input, p_es->p_pgrm->pp_es[i_loop] );
+            {
+                input_DelES( p_input, p_es->p_pgrm->pp_es[i_loop] );
+            }
         }
 
         /* Then add what we received in this PMT */
         do
         {
-            i_section_length = ( (*(p_current_data + 1) & 0xF) << 8 ) |
+            i_section_length = ( ((u32)*(p_current_data + 1) & 0xF) << 8 ) |
                                   *(p_current_data + 2);
             i_current_section = (u8)p_current_data[6];
-            i_prog_info_length = ( (*(p_current_data + 10) & 0xF) << 8 ) |
+            i_prog_info_length = ( ((u32)*(p_current_data + 10) & 0xF) << 8 ) |
                                     *(p_current_data + 11);
 
             /* For the moment we ignore program descriptors */
@@ -1478,9 +1536,9 @@ static void input_DecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
             while( p_current_data < p_current_section + i_section_length -1 )
             {
                 i_stream_type = (int)p_current_data[0];
-                i_pid = ( (*(p_current_data + 1) & 0x1F) << 8 ) |
+                i_pid = ( ((u32)*(p_current_data + 1) & 0x1F) << 8 ) |
                            *(p_current_data + 2);
-                i_es_info_length = ( (*(p_current_data + 3) & 0xF) << 8 ) |
+                i_es_info_length = ( ((u32)*(p_current_data + 3) & 0xF) << 8 ) |
                                       *(p_current_data + 4);
                 
                 /* Add this ES to the program */
