@@ -3,8 +3,8 @@
  * This library provides basic functions for threads to interact with user
  * interface, such as command line.
  *****************************************************************************
- * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: interface.c,v 1.80 2001/07/18 14:21:00 massiot Exp $
+ * Copyright (C) 1998-2001 VideoLAN
+ * $Id: interface.c,v 1.82 2001/12/07 18:33:08 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -34,8 +34,8 @@
 #include <string.h>                                            /* strerror() */
 #include <sys/types.h>                                              /* off_t */
 
-#include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "modules.h"
@@ -45,7 +45,6 @@
 
 #include "audio_output.h"
 
-#include "intf_msg.h"
 #include "interface.h"
 #include "intf_playlist.h"
 #include "intf_channels.h"
@@ -54,8 +53,6 @@
 #include "video.h"
 #include "video_output.h"
 
-#include "main.h"
-
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
