@@ -3,8 +3,8 @@
  * This library provides basic functions for threads to interact with user
  * interface, such as message output. See config.h for output configuration.
  *****************************************************************************
- * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: intf_msg.c,v 1.32 2001/04/28 03:36:25 sam Exp $
+ * Copyright (C) 1998-2001 VideoLAN
+ * $Id: intf_msg.c,v 1.40 2001/12/07 18:33:08 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -34,22 +34,17 @@
 #include <stdarg.h>                                       /* va_list for BSD */
 #include <stdlib.h>                                              /* malloc() */
 #include <string.h>                                            /* strerror() */
-#include <unistd.h>                                      /* close(), write() */
 
-#include "config.h"
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>                                      /* close(), write() */
+#endif
+
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 
-#include "intf_msg.h"
 #include "interface.h"
-#include "intf_console.h"
-
-#include "main.h"
-
-#ifdef WIN32
-#define snprintf _snprintf         /* snprintf not defined in mingw32 (bug?) */
-#endif
 
 /*****************************************************************************
  * intf_msg_item_t
@@ -65,7 +60,7 @@ typedef struct
 
 #ifdef TRACE
     /* Debugging informations - in TRACE mode, debug messages have calling
-     * location informations printed */
+     * location information printed */
     mtime_t date;                                     /* date of the message */
     char *  psz_file;               /* file in which the function was called */
     char *  psz_function;     /* function from which the function was called */
@@ -76,9 +71,9 @@ typedef struct
 /* Message types */
 #define INTF_MSG_STD    0                                /* standard message */
 #define INTF_MSG_ERR    1                                   /* error message */
-#define INTF_MSG_INTF   2                               /* interface message */
 #define INTF_MSG_DBG    3                                   /* debug message */
-#define INTF_MSG_WARN   4                                  /* warning message*/
+#define INTF_MSG_WARN   4                                 /* warning message */
+#define INTF_MSG_STAT   5                               /* statistic message */
 
 
 /*****************************************************************************
@@ -125,6 +120,9 @@ static void QueueDbgMsg     ( intf_msg_t *p_msg, char *psz_file,
 static void FlushLockedMsg  ( intf_msg_t *p_msg );
 #endif
 
+#if defined( WIN32 )
+static char *ConvertPrintfFormatString ( char *psz_format );
+#endif
 
 /*****************************************************************************
  * intf_MsgCreate: initialize messages interface                         (ok ?)
@@ -229,7 +227,7 @@ void intf_WarnMsg( int i_level, char *psz_format, ... )
 {
     va_list ap;
     
-    if( i_level >= p_main->i_warning_level )
+    if( i_level <= p_main->i_warning_level )
     {
         va_start( ap, psz_format );
         QueueMsg( p_main->p_msg, INTF_MSG_WARN, psz_format, ap );
@@ -237,25 +235,22 @@ void intf_WarnMsg( int i_level, char *psz_format, ... )
     }
 }
 
-
 /*****************************************************************************
- * intf_IntfMsg : print an interface message                             (ok ?)
+ * intf_StatMsg : print a statistic message
  *****************************************************************************
- * In opposition to all other intf_*Msg function, this function does not print
- * it's message on default terminal (stdout or stderr), but send it to
- * interface (in fact to the X11 console). This means that the interface MUST
- * be initialized and a X11 console openned before this function is used, and
- * that once the console is closed, this call is vorbidden.
- * Practically, only the interface thread itself should call this function, and
- * flush all messages before intf_CloseX11Console() is called.
+ * This function is the same as intf_Msg, except that it concerns statistic
+ * messages for testing purpose.
  *****************************************************************************/
-void intf_IntfMsg(char *psz_format, ...)
+void intf_StatMsg( char *psz_format, ... )
 {
     va_list ap;
-
-    va_start( ap, psz_format );
-    QueueMsg( p_main->p_msg, INTF_MSG_INTF, psz_format, ap );
-    va_end( ap );
+    
+    if( p_main->b_stats )
+    {
+        va_start( ap, psz_format );
+        QueueMsg( p_main->p_msg, INTF_MSG_STAT, psz_format, ap );
+        va_end( ap );
+    }
 }
 
 /*****************************************************************************
@@ -321,7 +316,7 @@ void intf_WarnMsgImm( int i_level, char *psz_format, ... )
 {
     va_list ap;
 
-    if( i_level >= p_main->i_warning_level )
+    if( i_level <= p_main->i_warning_level )
     {
         va_start( ap, psz_format );
         QueueMsg( p_main->p_msg, INTF_MSG_WARN, psz_format, ap );
@@ -421,6 +416,9 @@ static void QueueMsg( intf_msg_t *p_msg, int i_type, char *psz_format, va_list a
 {
     char *                  psz_str;             /* formatted message string */
     intf_msg_item_t *       p_msg_item;                /* pointer to message */
+#ifdef WIN32
+    char *                  psz_temp;
+#endif
 
 #ifndef INTF_MSG_QUEUE /*................................... instant mode ...*/
     intf_msg_item_t         msg_item;                             /* message */
@@ -430,6 +428,7 @@ static void QueueMsg( intf_msg_t *p_msg, int i_type, char *psz_format, va_list a
     /*
      * Convert message to string
      */
+
 #ifdef HAVE_VASPRINTF
     vasprintf( &psz_str, psz_format, ap );
 #else
@@ -443,10 +442,15 @@ static void QueueMsg( intf_msg_t *p_msg, int i_type, char *psz_format, va_list a
         fprintf(stderr, "\n" );
         exit( errno );
     }
-#ifdef HAVE_VASPRINTF
+#ifndef HAVE_VASPRINTF
+#ifdef WIN32
+    psz_temp = ConvertPrintfFormatString(psz_format);
+    vsprintf( psz_str, psz_temp, ap );
+    free( psz_temp );
 #else
     vsprintf( psz_str, psz_format, ap );
-#endif
+#endif /* WIN32 */
+#endif /* HAVE_VASPRINTF */
 
 #ifdef INTF_MSG_QUEUE /*...................................... queue mode ...*/
     vlc_mutex_lock( &p_msg->lock );                              /* get lock */
@@ -489,6 +493,9 @@ static void QueueDbgMsg(intf_msg_t *p_msg, char *psz_file, char *psz_function,
 {
     char *                  psz_str;             /* formatted message string */
     intf_msg_item_t *       p_msg_item;                /* pointer to message */
+#ifdef WIN32
+    char *                  psz_temp;
+#endif
 
 #ifndef INTF_MSG_QUEUE /*................................... instant mode ...*/
     intf_msg_item_t         msg_item;                             /* message */
@@ -502,7 +509,6 @@ static void QueueDbgMsg(intf_msg_t *p_msg, char *psz_file, char *psz_function,
     vasprintf( &psz_str, psz_format, ap );
 #else
     psz_str = (char*) malloc( INTF_MAX_MSG_SIZE );
-    vsprintf( psz_str, psz_format, ap );
 #endif
     if( psz_str == NULL )
     {
@@ -513,6 +519,15 @@ static void QueueDbgMsg(intf_msg_t *p_msg, char *psz_file, char *psz_function,
         fprintf(stderr, "\n" );
         exit( errno );
     }
+#ifndef HAVE_VASPRINTF
+#ifdef WIN32
+    psz_temp = ConvertPrintfFormatString(psz_format);
+    vsprintf( psz_str, psz_temp, ap );
+    free( psz_temp );
+#else
+    vsprintf( psz_str, psz_format, ap );
+#endif /* WIN32 */
+#endif /* HAVE_VASPRINTF */
 
 #ifdef INTF_MSG_QUEUE /*...................................... queue mode ...*/
     vlc_mutex_lock( &p_msg->lock );                              /* get lock */
@@ -597,6 +612,7 @@ static void PrintMsg( intf_msg_item_t *p_msg )
     switch( p_msg->i_type )
     {
     case INTF_MSG_STD:                                   /* regular messages */
+    case INTF_MSG_STAT:
     case INTF_MSG_ERR:
         snprintf( psz_msg, i_msg_len, "%s", p_msg->psz_msg );
         break;
@@ -608,10 +624,6 @@ static void PrintMsg( intf_msg_item_t *p_msg )
 
         break;
         
-    case INTF_MSG_INTF:                                /* interface messages */
-        snprintf( psz_msg, i_msg_len, "%s", p_msg->psz_msg );
-        break;
-
     case INTF_MSG_DBG:                                     /* debug messages */
         mstrtime( psz_date, p_msg->date );
         snprintf( psz_msg, i_msg_len, "(%s) " INTF_MSG_DBG_FORMAT "%s",
@@ -626,6 +638,7 @@ static void PrintMsg( intf_msg_item_t *p_msg )
     switch( p_msg->i_type )
     {
     case INTF_MSG_STD:                                  /* standard messages */
+    case INTF_MSG_STAT:
         fprintf( stdout, "%s\n", psz_msg );
         break;
     case INTF_MSG_ERR:                                     /* error messages */
@@ -635,9 +648,6 @@ static void PrintMsg( intf_msg_item_t *p_msg )
 #endif
         fprintf( stderr, "%s\n", psz_msg );
         break;
-    case INTF_MSG_INTF:                                /* interface messages */
-        intf_ConsolePrint( p_main->p_intf->p_console, psz_msg );
-        break;
     }
 
 #ifdef TRACE_LOG
@@ -663,6 +673,7 @@ static void PrintMsg( intf_msg_item_t *p_msg )
     switch( p_msg->i_type )
     {
     case INTF_MSG_STD:                                  /* standard messages */
+    case INTF_MSG_STAT:
     case INTF_MSG_DBG:                                     /* debug messages */
         fprintf( stdout, "%s\n", p_msg->psz_msg );
         break;
@@ -670,11 +681,63 @@ static void PrintMsg( intf_msg_item_t *p_msg )
     case INTF_MSG_WARN:
         fprintf( stderr, "%s\n", p_msg->psz_msg );        /* warning message */
         break;
-    case INTF_MSG_INTF:                                /* interface messages */
-        intf_ConsolePrint( p_main->p_intf->p_console, p_msg->psz_msg );
-        break;
     }
 }
 
 #endif
 
+
+#if defined( WIN32 )
+/*****************************************************************************
+ * ConvertPrintfFormatString: replace all occurrences of %ll with %I64 in the
+ *                            printf format string.
+ *****************************************************************************
+ * Win32 doesn't recognize the "%lld" format in a printf string, so we have
+ * to convert this string to something that win32 can handle.
+ * This is a REALLY UGLY HACK which won't even work in every situation,
+ * but hey I don't want to put an ifdef WIN32 each time I use printf with
+ * a "long long" type!!!
+ * By the way, if we don't do this we can sometimes end up with segfaults.
+ *****************************************************************************/
+static char *ConvertPrintfFormatString( char *psz_format )
+{
+  int i, i_counter=0, i_pos=0;
+  char *psz_dest;
+
+  /* We first need to check how many occurences of %ll there are in the
+   * psz_format string. Once we'll know that we'll be able to malloc the
+   * destination string */
+
+  for( i=0; i <= (strlen(psz_format) - 4); i++ )
+  {
+      if( !strncmp( (char *)(psz_format + i), "%ll", 3 ) )
+	  i_counter++;
+  }
+
+  /* malloc the destination string */
+  psz_dest = malloc( strlen(psz_format) + i_counter + 1 );
+  if( psz_dest == NULL )
+  {
+      fprintf(stderr, "warning: malloc failed in ConvertPrintfFormatString\n");
+      exit (errno);
+  }
+
+  /* Now build the modified string */
+  i_counter = 0;
+  for( i=0; i <= (strlen(psz_format) - 4); i++ )
+  {
+      if( !strncmp( (char *)(psz_format + i), "%ll", 3 ) )
+      {
+          memcpy( psz_dest+i_pos+i_counter, psz_format+i_pos, i-i_pos+1);
+          *(psz_dest+i+i_counter+1)='I';
+          *(psz_dest+i+i_counter+2)='6';
+          *(psz_dest+i+i_counter+3)='4';
+	  i_pos = i+3;
+          i_counter++;
+      }
+  }
+  strcpy( psz_dest+i_pos+i_counter, psz_format+i_pos );
+
+  return psz_dest;
+}
+#endif
