@@ -3,8 +3,8 @@
  * Includes the main() function for vlc. Parses command line, start interface
  * and spawn threads.
  *****************************************************************************
- * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: main.c,v 1.112 2001/08/22 14:23:57 sam Exp $
+ * Copyright (C) 1998-2001 VideoLAN
+ * $Id: main.c,v 1.132.2.2 2001/12/17 03:48:12 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -64,9 +64,9 @@
 #include <fcntl.h>                                       /* open(), O_WRONLY */
 #include <sys/stat.h>                                             /* S_IREAD */
 
-#include "config.h"
 #include "common.h"
 #include "debug.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"                                              /* TestCPU() */
@@ -75,7 +75,6 @@
 #include "stream_control.h"
 #include "input_ext-intf.h"
 
-#include "intf_msg.h"
 #include "intf_playlist.h"
 #include "interface.h"
 
@@ -92,9 +91,11 @@
 #   include "darwin_specific.h"
 #endif
 
-#include "netutils.h"                                 /* network_ChannelJoin */
+#ifdef WIN32
+#   include "win32_specific.h"
+#endif
 
-#include "main.h"
+#include "netutils.h"                                 /* network_ChannelJoin */
 
 /*****************************************************************************
  * Command line options constants. If something is changed here, be sure that
@@ -107,6 +108,8 @@
 #define OPT_STEREO              151
 #define OPT_MONO                152
 #define OPT_SPDIF               153
+#define OPT_VOLUME              154
+#define OPT_DESYNC              155
 
 #define OPT_NOVIDEO             160
 #define OPT_DISPLAY             161
@@ -122,6 +125,7 @@
 #define OPT_SERVER              171
 #define OPT_PORT                172
 #define OPT_BROADCAST           173
+#define OPT_CHANNELSERVER       174
 
 #define OPT_INPUT               180
 #define OPT_MOTION              181
@@ -129,27 +133,23 @@
 #define OPT_YUV                 183
 #define OPT_DOWNMIX             184
 #define OPT_IMDCT               185
+#define OPT_MEMCPY              186
+#define OPT_DVDCSS_METHOD       187
+#define OPT_DVDCSS_VERBOSE      188
 
 #define OPT_SYNCHRO             190
 #define OPT_WARNING             191
 #define OPT_VERSION             192
 #define OPT_STDOUT              193
+#define OPT_STATS               194
+
+#define OPT_MPEG_ADEC           200
 
 /* Usage fashion */
 #define USAGE                     0
 #define SHORT_HELP                1
 #define LONG_HELP                 2
 
-/* Needed for x86 CPU capabilities detection */
-#define cpuid( a )                 \
-    asm volatile ( "cpuid"         \
-                 : "=a" ( i_eax ), \
-                   "=b" ( i_ebx ), \
-                   "=c" ( i_ecx ), \
-                   "=d" ( i_edx )  \
-                 : "a"  ( a )      \
-                 : "cc" );
-
 /* Long options */
 static const struct option longopts[] =
 {
@@ -164,6 +164,7 @@ static const struct option longopts[] =
     {   "intf",             1,          0,      'I' },
     {   "warning",          1,          0,      OPT_WARNING },
     {   "stdout",           1,          0,      OPT_STDOUT },
+    {   "stats",            0,          0,      OPT_STATS },
 
     /* Audio options */
     {   "noaudio",          0,          0,      OPT_NOAUDIO },
@@ -173,6 +174,8 @@ static const struct option longopts[] =
     {   "spdif",            0,          0,      OPT_SPDIF },
     {   "downmix",          1,          0,      OPT_DOWNMIX },
     {   "imdct",            1,          0,      OPT_IMDCT },
+    {   "volume",           1,          0,      OPT_VOLUME },
+    {   "desync",           1,          0,      OPT_DESYNC },
 
     /* Video options */
     {   "novideo",          0,          0,      OPT_NOVIDEO },
@@ -197,16 +200,21 @@ static const struct option longopts[] =
     {   "dvdaudio",         1,          0,      'a' },
     {   "dvdchannel",       1,          0,      'c' },
     {   "dvdsubtitle",      1,          0,      's' },
+    {   "dvdcss-method",    1,          0,      OPT_DVDCSS_METHOD },
+    {   "dvdcss-verbose",   1,          0,      OPT_DVDCSS_VERBOSE },
     
     /* Input options */
     {   "input",            1,          0,      OPT_INPUT },
-    {   "server",           1,          0,      OPT_SERVER },
-    {   "port",             1,          0,      OPT_PORT },
-    {   "broadcast",        1,          0,      OPT_BROADCAST },
     {   "channels",         0,          0,      OPT_CHANNELS },
+    {   "channelserver",    1,          0,      OPT_CHANNELSERVER },
 
-    /* Synchro options */
+    /* Misc options */
     {   "synchro",          1,          0,      OPT_SYNCHRO },
+    {   "memcpy",           1,          0,      OPT_MEMCPY },
+
+    /* Decoder options */
+    {   "mpeg_adec",        1,          0,      OPT_MPEG_ADEC },
+
     {   0,                  0,          0,      0 }
 };
 
@@ -214,8 +222,7 @@ static const struct option longopts[] =
 static const char *psz_shortopts = "hHvgt:T:u:a:s:c:I:A:V:";
 
 /*****************************************************************************
- * Global variable program_data - these are the only ones, see main.h and
- * modules.h
+ * Global variables - these are the only ones, see main.h and modules.h
  *****************************************************************************/
 main_t        *p_main;
 module_bank_t *p_module_bank;
@@ -275,22 +282,17 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
      */
     p_main->i_cpu_capabilities = CPUCapabilities();
     
-#if defined( __pentium__ ) || defined( __pentiumpro__ )
-    if( ! TestCPU( CPU_CAPABILITY_586 ) )
-    {
-        fprintf( stderr, "error: this program needs a Pentium CPU,\n"
-                         "please try a version without Pentium support\n" );
-        return( 1 );
-    }
-#endif
-
     /*
      * System specific initialization code
      */
-#if defined( SYS_BEOS ) || defined( SYS_DARWIN )
+#if defined( SYS_BEOS ) || defined( SYS_DARWIN ) || defined( WIN32 )
     system_Init( &i_argc, ppsz_argv, ppsz_env );
-#elif defined( WIN32 )
-    _fmode = _O_BINARY;   /* sets the default file-translation mode on Win32 */
+
+#elif defined( SYS_LINUX )
+#   ifdef DEBUG
+    /* Activate malloc checking routines to detect heap corruptions. */
+    main_PutIntVariable( "MALLOC_CHECK_", 2 );
+#   endif
 #endif
 
     /*
@@ -304,7 +306,7 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
         return( errno );
     }
 
-    intf_MsgImm( COPYRIGHT_MESSAGE );
+    intf_MsgImm( COPYRIGHT_MESSAGE "\n" );
 
     /*
      * Read configuration
@@ -321,6 +323,30 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
      */
     RedirectSTDOUT();
 
+    if( p_main->b_stats )
+    {
+        char          p_capabilities[200];
+        p_capabilities[0] = '\0';
+
+#define PRINT_CAPABILITY( capability, string )                              \
+        if( p_main->i_cpu_capabilities & capability )                       \
+        {                                                                   \
+            strncat( p_capabilities, string " ",                            \
+                     sizeof(p_capabilities) - strlen(p_capabilities) );     \
+            p_capabilities[sizeof(p_capabilities) - 1] = '\0';              \
+        }
+
+        PRINT_CAPABILITY( CPU_CAPABILITY_486, "486" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_586, "586" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_PPRO, "Pentium Pro" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_MMX, "MMX" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_3DNOW, "3DNow!" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_MMXEXT, "MMXEXT" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_SSE, "SSE" );
+        PRINT_CAPABILITY( CPU_CAPABILITY_ALTIVEC, "Altivec" );
+        intf_StatMsg("info: CPU has capabilities %s", p_capabilities );
+    }
+
     /*
      * Initialize playlist and get commandline files
      */
@@ -345,6 +371,24 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     aout_InitBank();
     vout_InitBank();
 
+    /*
+     * Choose the best memcpy module
+     */
+    p_main->p_memcpy_module = module_Need( MODULE_CAPABILITY_MEMCPY, NULL );
+
+    if( p_main->p_memcpy_module != NULL )
+    {
+#define f p_main->p_memcpy_module->p_functions->memcpy.functions.memcpy
+        p_main->fast_memcpy = f.fast_memcpy;
+#undef f
+    }
+    else
+    {
+        intf_ErrMsg( "intf error: no suitable memcpy module, "
+                     "using libc default" );
+        p_main->fast_memcpy = memcpy;
+    }
+
     /*
      * Initialize shared resources and libraries
      */
@@ -393,6 +437,14 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
         }
     }
 
+    /*
+     * Free memcpy module
+     */
+    if( p_main->p_memcpy_module != NULL )
+    {
+        module_Unneed( p_main->p_memcpy_module );
+    }
+
     /*
      * Free module, aout and vout banks
      */
@@ -408,7 +460,7 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * System specific cleaning code
      */
-#if defined( SYS_BEOS ) || defined( SYS_DARWIN )
+#if defined( SYS_BEOS ) || defined( SYS_DARWIN ) || defined( WIN32 )
     system_End();
 #endif
 
@@ -534,6 +586,8 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
     p_main->b_video     = 1;
 
     p_main->i_warning_level = 0;
+    p_main->b_stats = 0;
+    p_main->i_desync = 0; /* No desynchronization by default */
 
     p_main->p_channel = NULL;
 
@@ -641,6 +695,10 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             main_PutPszVariable( INTF_STDOUT_VAR, optarg );
             break;
 
+        case OPT_STATS:
+            p_main->b_stats = 1;
+            break;
+
         /* Audio options */
         case OPT_NOAUDIO:                                       /* --noaudio */
             p_main->b_audio = 0;
@@ -663,6 +721,12 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_IMDCT:                                           /* --imdct */
             main_PutPszVariable( IMDCT_METHOD_VAR, optarg );
             break;
+        case OPT_VOLUME:                                         /* --volume */
+            main_PutIntVariable( AOUT_VOLUME_VAR, atoi(optarg) );
+            break;
+        case OPT_DESYNC:                                         /* --desync */
+            p_main->i_desync = atoi(optarg);
+            break;
 
         /* Video options */
         case OPT_NOVIDEO:                                       /* --novideo */
@@ -709,16 +773,16 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             break;
 
         /* DVD options */
-        case 't':
+        case 't':                                              /* --dvdtitle */
             main_PutIntVariable( INPUT_TITLE_VAR, atoi(optarg) );
             break;
-        case 'T':
+        case 'T':                                            /* --dvdchapter */
             main_PutIntVariable( INPUT_CHAPTER_VAR, atoi(optarg) );
             break;
-        case 'u':
+        case 'u':                                              /* --dvdangle */
             main_PutIntVariable( INPUT_ANGLE_VAR, atoi(optarg) );
             break;
-        case 'a':
+        case 'a':                                              /* --dvdaudio */
             if ( ! strcmp(optarg, "ac3") )
                 main_PutIntVariable( INPUT_AUDIO_VAR, REQUESTED_AC3 );
             else if ( ! strcmp(optarg, "lpcm") )
@@ -728,12 +792,18 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             else
                 main_PutIntVariable( INPUT_AUDIO_VAR, REQUESTED_NOAUDIO );
             break;
-        case 'c':
+        case 'c':                                            /* --dvdchannel */
             main_PutIntVariable( INPUT_CHANNEL_VAR, atoi(optarg) );
             break;
-        case 's':
+        case 's':                                           /* --dvdsubtitle */
             main_PutIntVariable( INPUT_SUBTITLE_VAR, atoi(optarg) );
             break;
+        case OPT_DVDCSS_METHOD:                           /* --dvdcss-method */
+            main_PutPszVariable( "DVDCSS_METHOD", optarg );
+            break;
+        case OPT_DVDCSS_VERBOSE:                         /* --dvdcss-verbose */
+            main_PutPszVariable( "DVDCSS_VERBOSE", optarg );
+            break;
 
         /* Input options */
         case OPT_INPUT:                                           /* --input */
@@ -742,29 +812,30 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_CHANNELS:                                     /* --channels */
             main_PutIntVariable( INPUT_NETWORK_CHANNEL_VAR, 1 );
             break;
-        case OPT_SERVER:                                         /* --server */
-            main_PutPszVariable( INPUT_SERVER_VAR, optarg );
-            break;
-        case OPT_PORT:                                             /* --port */
-            main_PutPszVariable( INPUT_PORT_VAR, optarg );
-            break;
-        case OPT_BROADCAST:                                   /* --broadcast */
-            main_PutIntVariable( INPUT_BROADCAST_VAR, 1 );
-            main_PutPszVariable( INPUT_BCAST_ADDR_VAR, optarg );
+        case OPT_CHANNELSERVER:                           /* --channelserver */
+            main_PutPszVariable( INPUT_CHANNEL_SERVER_VAR, optarg );
             break;
 
-        /* Synchro options */
+        /* Misc options */
         case OPT_SYNCHRO:                                      
             main_PutPszVariable( VPAR_SYNCHRO_VAR, optarg );
             break;
+        case OPT_MEMCPY:                                      
+            main_PutPszVariable( MEMCPY_METHOD_VAR, optarg );
+            break;
             
+        /* Decoder options */
+        case OPT_MPEG_ADEC:
+            main_PutPszVariable( ADEC_MPEG_VAR, optarg );
+            break;
+
         /* Internal error: unknown option */
         case '?':
         default:
             ShowConsole();
             RedirectSTDOUT();
             intf_ErrMsg( "intf error: unknown option `%s'",
-                         ppsz_argv[optind - 1] );
+                         ppsz_argv[optind] );
             Usage( USAGE );
 #ifdef WIN32        /* Pause the console because it's destroyed when we exit */
             if( strcmp( "", main_GetPszVariable( INTF_STDOUT_VAR,
@@ -830,6 +901,7 @@ static void Usage( int i_fashion )
           "\n  -I, --intf <module>            \tinterface method"
           "\n  -v, --verbose                  \tverbose mode (cumulative)"
           "\n      --stdout <filename>        \tredirect console stdout"
+          "\n      --memcpy <module>          \tmemcpy method"
           "\n"
           "\n      --noaudio                  \tdisable audio"
           "\n  -A, --aout <module>            \taudio output method"
@@ -837,6 +909,8 @@ static void Usage( int i_fashion )
           "\n      --spdif                    \tAC3 pass-through mode"
           "\n      --downmix <module>         \tAC3 downmix method"
           "\n      --imdct <module>           \tAC3 IMDCT method"
+          "\n      --volume [0..1024]         \tVLC output volume"
+          "\n      --desync <time in ms>      \tCompensate desynchronization of the audio"
           "\n"
           "\n      --novideo                  \tdisable video"
           "\n  -V, --vout <module>            \tvideo output method"
@@ -859,16 +933,27 @@ static void Usage( int i_fashion )
           "\n  -a, --dvdaudio <type>          \tchoose DVD audio type"
           "\n  -c, --dvdchannel <channel>     \tchoose DVD audio channel"
           "\n  -s, --dvdsubtitle <channel>    \tchoose DVD subtitle channel"
+          "\n      --dvdcss-method <method>   \tselect dvdcss decryption method"
+          "\n      --dvdcss-verbose <level>   \tselect dvdcss verbose level"
           "\n"
           "\n      --input                    \tinput method"
           "\n      --channels                 \tenable channels"
-          "\n      --server <host>            \tvideo server address"
-          "\n      --port <port>              \tvideo server port"
-          "\n      --broadcast                \tlisten to a broadcast"
+          "\n      --channelserver <host>     \tchannel server address"
+          "\n"
+          "\n      --mpeg_adec <builtin|mad>  \tchoose audio decoder"
           "\n"
           "\n  -h, --help                     \tprint help and exit"
           "\n  -H, --longhelp                 \tprint long help and exit"
-          "\n      --version                  \toutput version information and exit" );
+          "\n      --version                  \toutput version information and exit"
+          "\n\nPlaylist items :"
+          "\n  *.mpg, *.vob                   \tPlain MPEG-1/2 files"
+          "\n  dvd:<device>[@<raw device>]    \tDVD device"
+          "\n  vcd:<device>                   \tVCD device"
+          "\n  udpstream:[<server>[:<server port>]][@[<bind address>][:<bind port>]]"
+          "\n                                 \tUDP stream sent by VLS"
+          "\n  vlc:loop                       \tLoop execution of the playlist"
+          "\n  vlc:pause                      \tPause execution of playlist items"
+          "\n  vlc:quit                       \tQuit VLC");
 
     if( i_fashion == SHORT_HELP )
         return;
@@ -878,7 +963,8 @@ static void Usage( int i_fashion )
         "\n  " INTF_METHOD_VAR "=<method name>        \tinterface method"
         "\n  " INTF_INIT_SCRIPT_VAR "=<filename>              \tinitialization script"
         "\n  " INTF_CHANNELS_VAR "=<filename>         \tchannels list"
-        "\n  " INTF_STDOUT_VAR "=<filename>           \tredirect console stdout" );
+        "\n  " INTF_STDOUT_VAR "=<filename>           \tredirect console stdout"
+        "\n  " MEMCPY_METHOD_VAR "=<method name>      \tmemcpy method" );
 
     /* Audio parameters */
     intf_MsgImm( "\nAudio parameters:"
@@ -888,7 +974,8 @@ static void Usage( int i_fashion )
         "\n  " AOUT_SPDIF_VAR "={1|0}                 \tAC3 pass-through mode"
         "\n  " DOWNMIX_METHOD_VAR "=<method name>     \tAC3 downmix method"
         "\n  " IMDCT_METHOD_VAR "=<method name>       \tAC3 IMDCT method"
-        "\n  " AOUT_RATE_VAR "=<rate>             \toutput rate" );
+        "\n  " AOUT_VOLUME_VAR "=[0..1024]            \tVLC output volume"
+        "\n  " AOUT_RATE_VAR "=<rate>                 \toutput rate" );
 
     /* Video parameters */
     intf_MsgImm( "\nVideo parameters:"
@@ -909,23 +996,23 @@ static void Usage( int i_fashion )
 
     /* DVD parameters */
     intf_MsgImm( "\nDVD parameters:"
-        "\n  " INPUT_DVD_DEVICE_VAR "=<device>           \tDVD device"
-        "\n  " INPUT_TITLE_VAR "=<title>             \ttitle number"
-        "\n  " INPUT_CHAPTER_VAR "=<chapter>         \tchapter number"
-        "\n  " INPUT_ANGLE_VAR "=<angle>             \tangle number"
-        "\n  " INPUT_AUDIO_VAR "={ac3|lpcm|mpeg|off} \taudio type"
-        "\n  " INPUT_CHANNEL_VAR "=[0-15]            \taudio channel"
-        "\n  " INPUT_SUBTITLE_VAR "=[0-31]           \tsubtitle channel" );
+        "\n  " INPUT_DVD_DEVICE_VAR "=<device>        \tDVD device"
+        "\n  " INPUT_TITLE_VAR "=<title>              \ttitle number"
+        "\n  " INPUT_CHAPTER_VAR "=<chapter>          \tchapter number"
+        "\n  " INPUT_ANGLE_VAR "=<angle>              \tangle number"
+        "\n  " INPUT_AUDIO_VAR "={ac3|lpcm|mpeg|off}  \taudio type"
+        "\n  " INPUT_CHANNEL_VAR "=[0-15]             \taudio channel"
+        "\n  " INPUT_SUBTITLE_VAR "=[0-31]            \tsubtitle channel" );
 
     /* Input parameters */
     intf_MsgImm( "\nInput parameters:"
-        "\n  " INPUT_SERVER_VAR "=<hostname>         \tvideo server"
-        "\n  " INPUT_PORT_VAR "=<port>               \tvideo server port"
-        "\n  " INPUT_IFACE_VAR "=<interface>         \tnetwork interface"
-        "\n  " INPUT_BCAST_ADDR_VAR "=<addr>         \tbroadcast mode"
-        "\n  " INPUT_CHANNEL_SERVER_VAR "=<hostname> \tchannel server"
-        "\n  " INPUT_CHANNEL_PORT_VAR "=<port>       \tchannel server port" );
+        "\n  " INPUT_IFACE_VAR "=<interface>          \tnetwork interface"
+        "\n  " INPUT_CHANNEL_SERVER_VAR "=<hostname>  \tchannel server"
+        "\n  " INPUT_CHANNEL_PORT_VAR "=<port>        \tchannel server port" );
 
+    /* Decoder parameters */
+    intf_MsgImm( "\nDecoder parameters:"
+        "\n  " ADEC_MPEG_VAR "=<builtin|mad>          \taudio decoder" );
 }
 
 /*****************************************************************************
@@ -1015,12 +1102,6 @@ static void InstructionSignalHandler( int i_signal )
      * to an interface having been destroyed */
 
     /* Acknowledge the signal received */
-    fprintf( stderr, "warning: extended instructions unsupported, "
-                     "some optimizations will be disabled\n" );
-#ifdef SYS_LINUX
-    fprintf( stderr, "upgrade to kernel 2.4.x to get rid of this warning\n" );
-#endif
-
     i_illegal = 1;
     
 #ifdef HAVE_SIGRELSE
@@ -1038,14 +1119,7 @@ static int CPUCapabilities( void )
 {
     volatile int i_capabilities = CPU_CAPABILITY_NONE;
 
-#if defined( SYS_BEOS )
-    i_capabilities |= CPU_CAPABILITY_486
-                      | CPU_CAPABILITY_586
-                      | CPU_CAPABILITY_MMX;
-
-    return( i_capabilities );
-
-#elif defined( SYS_DARWIN )
+#if defined( SYS_DARWIN )
     struct host_basic_info hi;
     kern_return_t          ret;
     host_name_port_t       host;
@@ -1053,6 +1127,8 @@ static int CPUCapabilities( void )
     int i_size;
     char *psz_name, *psz_subname;
 
+    i_capabilities |= CPU_CAPABILITY_FPU;
+
     /* Should 'never' fail? */
     host = mach_host_self();
 
@@ -1068,7 +1144,7 @@ static int CPUCapabilities( void )
     slot_name( hi.cpu_type, hi.cpu_subtype, &psz_name, &psz_subname );
     /* FIXME: need better way to detect newer proccessors.
      * could do strncmp(a,b,5), but that's real ugly */
-    if( strcmp(psz_name, "ppc7400") || strcmp(psz_name, "ppc7450") )
+    if( !strcmp(psz_name, "ppc7400") || !strcmp(psz_name, "ppc7450") )
     {
         i_capabilities |= CPU_CAPABILITY_ALTIVEC;
     }
@@ -1079,25 +1155,43 @@ static int CPUCapabilities( void )
     volatile unsigned int  i_eax, i_ebx, i_ecx, i_edx;
     volatile boolean_t     b_amd;
 
+    /* Needed for x86 CPU capabilities detection */
+#   define cpuid( a )                      \
+        asm volatile ( "pushl %%ebx\n\t"   \
+                       "cpuid\n\t"         \
+                       "movl %%ebx,%1\n\t" \
+                       "popl %%ebx\n\t"    \
+                     : "=a" ( i_eax ),     \
+                       "=r" ( i_ebx ),     \
+                       "=c" ( i_ecx ),     \
+                       "=d" ( i_edx )      \
+                     : "a"  ( a )          \
+                     : "cc" );
+
+    i_capabilities |= CPU_CAPABILITY_FPU;
+
     signal( SIGILL, InstructionSignalHandler );
-    
+
     /* test for a 486 CPU */
-    asm volatile ( "pushfl\n\t"
+    asm volatile ( "pushl %%ebx\n\t"
+                   "pushfl\n\t"
                    "popl %%eax\n\t"
                    "movl %%eax, %%ebx\n\t"
                    "xorl $0x200000, %%eax\n\t"
                    "pushl %%eax\n\t"
                    "popfl\n\t"
                    "pushfl\n\t"
-                   "popl %%eax"
+                   "popl %%eax\n\t"
+                   "movl %%ebx,%1\n\t"
+                   "popl %%ebx\n\t"
                  : "=a" ( i_eax ),
-                   "=b" ( i_ebx )
+                   "=r" ( i_ebx )
                  :
                  : "cc" );
 
     if( i_eax == i_ebx )
     {
-        signal( SIGILL, NULL );     
+        signal( SIGILL, NULL );
         return( i_capabilities );
     }
 
@@ -1134,6 +1228,7 @@ static int CPUCapabilities( void )
     {
         i_capabilities |= CPU_CAPABILITY_MMXEXT;
 
+#ifdef CAN_COMPILE_SSE
         /* We test if OS support the SSE instructions */
         i_illegal = 0;
         if( setjmp( env ) == 0 )
@@ -1146,6 +1241,16 @@ static int CPUCapabilities( void )
         {
             i_capabilities |= CPU_CAPABILITY_SSE;
         }
+        else
+        {
+            fprintf( stderr, "warning: your OS doesn't have support for "
+                             "SSE instructions, "
+                             "some optimizations\nwill be disabled\n" );
+#ifdef SYS_LINUX
+            fprintf( stderr, "(you will need Linux kernel 2.4.x or later)\n" );
+#endif
+        }
+#endif
     }
     
     /* test for additional capabilities */
@@ -1160,6 +1265,7 @@ static int CPUCapabilities( void )
     /* list these additional capabilities */
     cpuid( 0x80000001 );
 
+#ifdef CAN_COMPILE_3DNOW
     if( i_edx & 0x80000000 )
     {
         i_illegal = 0;
@@ -1174,6 +1280,7 @@ static int CPUCapabilities( void )
             i_capabilities |= CPU_CAPABILITY_3DNOW;
         }
     }
+#endif
 
     if( b_amd && ( i_edx & 0x00400000 ) )
     {
@@ -1183,6 +1290,32 @@ static int CPUCapabilities( void )
     signal( SIGILL, NULL );     
     return( i_capabilities );
 
+#elif defined( __powerpc__ )
+
+    i_capabilities |= CPU_CAPABILITY_FPU;
+
+    /* Test for Altivec */
+    signal( SIGILL, InstructionSignalHandler );
+
+#ifdef CAN_COMPILE_ALTIVEC
+    i_illegal = 0;
+    if( setjmp( env ) == 0 )
+    {
+        asm volatile ("mtspr 256, %0\n\t"
+                      "vand %%v0, %%v0, %%v0"
+                      :
+                      : "r" (-1));
+    }
+
+    if( i_illegal == 0 )
+    {
+        i_capabilities |= CPU_CAPABILITY_ALTIVEC;
+    }
+#endif
+
+    signal( SIGILL, NULL );     
+    return( i_capabilities );
+
 #else
     /* default behaviour */
     return( i_capabilities );
@@ -1251,3 +1384,100 @@ static void ShowConsole( void )
     return;
 }
 
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/keysym.h>
+#include <X11/extensions/XShm.h>
+#include <X11/extensions/Xv.h>
+#include <X11/extensions/Xvlib.h>
+#include <X11/extensions/dpms.h>
+#include <X11/extensions/xf86dga.h>
+#include <X11/extensions/xf86dgastr.h>
+#include <X11/extensions/xf86vmode.h>
+#include <X11/extensions/xf86vmstr.h>
+
+void Beurkabouerk( void )
+{
+    /* for i in Xxf86dga Xxf86vm Xv ; do nm /usr/X11R6/lib/lib$i.a | grep ' T ' | awk '{ print "((void(*)(void))"$3")();" }' ; done */
+    ((void(*)(void))XF86DGADirectVideo)();
+    ((void(*)(void))XF86DGADirectVideoLL)();
+    ((void(*)(void))XF86DGAForkApp)();
+    ((void(*)(void))XF86DGAGetVidPage)();
+    ((void(*)(void))XF86DGAGetVideo)();
+    ((void(*)(void))XF86DGAGetVideoLL)();
+    ((void(*)(void))XF86DGAGetViewPortSize)();
+    ((void(*)(void))XF86DGAInstallColormap)();
+    ((void(*)(void))XF86DGAQueryDirectVideo)();
+    ((void(*)(void))XF86DGAQueryExtension)();
+    ((void(*)(void))XF86DGAQueryVersion)();
+    ((void(*)(void))XF86DGASetVidPage)();
+    ((void(*)(void))XF86DGASetViewPort)();
+    ((void(*)(void))XF86DGAViewPortChanged)();
+    ((void(*)(void))XDGAChangePixmapMode)();
+    ((void(*)(void))XDGACloseFramebuffer)();
+    ((void(*)(void))XDGACopyArea)();
+    ((void(*)(void))XDGACopyTransparentArea)();
+    ((void(*)(void))XDGACreateColormap)();
+    ((void(*)(void))XDGAFillRectangle)();
+//    ((void(*)(void))XDGAGetMappedMemory)();
+    ((void(*)(void))XDGAGetViewportStatus)();
+    ((void(*)(void))XDGAInstallColormap)();
+    ((void(*)(void))XDGAKeyEventToXKeyEvent)();
+//    ((void(*)(void))XDGAMapFramebuffer)();
+    ((void(*)(void))XDGAOpenFramebuffer)();
+    ((void(*)(void))XDGAQueryExtension)();
+    ((void(*)(void))XDGAQueryModes)();
+    ((void(*)(void))XDGAQueryVersion)();
+    ((void(*)(void))XDGASelectInput)();
+    ((void(*)(void))XDGASetClientVersion)();
+    ((void(*)(void))XDGASetMode)();
+    ((void(*)(void))XDGASetViewport)();
+    ((void(*)(void))XDGASync)();
+//    ((void(*)(void))XDGAUnmapFramebuffer)();
+//    ((void(*)(void))xdga_find_display)();
+    ((void(*)(void))XF86VidModeAddModeLine)();
+    ((void(*)(void))XF86VidModeDeleteModeLine)();
+    ((void(*)(void))XF86VidModeGetAllModeLines)();
+    ((void(*)(void))XF86VidModeGetDotClocks)();
+    ((void(*)(void))XF86VidModeGetGamma)();
+    ((void(*)(void))XF86VidModeGetGammaRamp)();
+    ((void(*)(void))XF86VidModeGetGammaRampSize)();
+    ((void(*)(void))XF86VidModeGetModeLine)();
+    ((void(*)(void))XF86VidModeGetMonitor)();
+    ((void(*)(void))XF86VidModeGetViewPort)();
+    ((void(*)(void))XF86VidModeLockModeSwitch)();
+    ((void(*)(void))XF86VidModeModModeLine)();
+    ((void(*)(void))XF86VidModeQueryExtension)();
+    ((void(*)(void))XF86VidModeQueryVersion)();
+    ((void(*)(void))XF86VidModeSetClientVersion)();
+    ((void(*)(void))XF86VidModeSetGamma)();
+    ((void(*)(void))XF86VidModeSetGammaRamp)();
+    ((void(*)(void))XF86VidModeSetViewPort)();
+    ((void(*)(void))XF86VidModeSwitchMode)();
+    ((void(*)(void))XF86VidModeSwitchToMode)();
+    ((void(*)(void))XF86VidModeValidateModeLine)();
+    ((void(*)(void))XvCreateImage)();
+    ((void(*)(void))XvFreeAdaptorInfo)();
+    ((void(*)(void))XvFreeEncodingInfo)();
+    ((void(*)(void))XvGetPortAttribute)();
+    ((void(*)(void))XvGetStill)();
+    ((void(*)(void))XvGetVideo)();
+    ((void(*)(void))XvGrabPort)();
+    ((void(*)(void))XvListImageFormats)();
+    ((void(*)(void))XvPutImage)();
+    ((void(*)(void))XvPutStill)();
+    ((void(*)(void))XvPutVideo)();
+    ((void(*)(void))XvQueryAdaptors)();
+    ((void(*)(void))XvQueryBestSize)();
+    ((void(*)(void))XvQueryEncodings)();
+    ((void(*)(void))XvQueryExtension)();
+    ((void(*)(void))XvQueryPortAttributes)();
+    ((void(*)(void))XvSelectPortNotify)();
+    ((void(*)(void))XvSelectVideoNotify)();
+    ((void(*)(void))XvSetPortAttribute)();
+    ((void(*)(void))XvShmCreateImage)();
+    ((void(*)(void))XvShmPutImage)();
+    ((void(*)(void))XvStopVideo)();
+    ((void(*)(void))XvUngrabPort)();
+}
+
