@@ -2,7 +2,8 @@
  * video_yuv.c: YUV transformation functions
  * These functions set up YUV tables for colorspace conversion
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
+ * Copyright (C) 1999-2001 VideoLAN
+ * $Id: video_yuv.c,v 1.49 2001/11/28 15:08:06 massiot Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -34,6 +35,7 @@
 
 #include "config.h"
 #include "common.h"
+#include "intf_msg.h"
 #include "threads.h"
 #include "mtime.h"
 #include "modules.h"
@@ -42,10 +44,6 @@
 #include "video_output.h"
 #include "video_yuv.h"
 
-#include "intf_msg.h"
-
-#include "main.h"
-
 /*****************************************************************************
  * vout_InitYUV: allocate and initialize translation tables
  *****************************************************************************
@@ -55,8 +53,7 @@
 int vout_InitYUV( vout_thread_t *p_vout )
 {
     /* Choose the best module */
-    p_vout->yuv.p_module = module_Need( p_main->p_bank,
-                                        MODULE_CAPABILITY_YUV, NULL );
+    p_vout->yuv.p_module = module_Need( MODULE_CAPABILITY_YUV, NULL );
 
     if( p_vout->yuv.p_module == NULL )
     {
@@ -93,6 +90,6 @@ int vout_ResetYUV( vout_thread_t *p_vout )
 void vout_EndYUV( vout_thread_t *p_vout )
 {
     p_vout->yuv.pf_end( p_vout );
-    module_Unneed( p_main->p_bank, p_vout->yuv.p_module );
+    module_Unneed( p_vout->yuv.p_module );
 }
 
